#ifndef CLOTHPATCH_H
#define CLOTHPATCH_H

#include "Object.h"
#include "PSystem.h"
#include "GL/gl.h"

struct Tri
{
	Penguin::Particle* p1;
	Penguin::Particle* p2;
	Penguin::Particle* p3;
};

/*! \brief A simple Cloth Patch class.
*
* This class provides an easy to use cloth patch object.
*/
class ClothPatch: public Penguin::Object
{
	int m_Width; //in # of vertices

	std::vector<Penguin::DampedSpring*> m_StructuralSprings;
	std::vector<Penguin::DampedSpring*> m_ShearSprings;
	std::vector<Penguin::DampedSpring*> m_BendSprings;
	
	std::vector<Tri> m_Tris; //used to make my life easier...
	
	int m_CurrentDrawMode;
	
	void SetupPatch(double width, double str, double shear, double bend, double mass);
	
	void DrawStructs();
	void DrawShears();
	void DrawPoints();
	void DrawPolygons();
	
	public:
	
	/*! Creates a default 10x10 cloth patch. */	
	ClothPatch();
	
	/*! Creates an mxm cloth patch. With optional total width, structural, shear and bend parameters. */
	ClothPatch(int m, double width=1.0, double structuralStrength=1000.0, double shearStrength=100.0, double bendStrength=100.0, double mass = 0.8);
	
	~ClothPatch();
	
	/*! Returns a reference to the particle ptr at the i'th row, j'th column. Returns the Particle at 1,1 if i and j are out of bounds */
	Penguin::Particle*& GetParticle(int i, int j);	
		
	/*! Draws this patch. */
	void Draw();	
	
	void NextDrawMode(); 
	
	/* Define Object interface. */
	virtual void SatisfyConstraints(){};		
	virtual void ApplyForces();
	virtual void SimulateParametricParticles(double t){};
	virtual void Update(double dt){}	
	
	virtual bool HasInternalCollisions(){return false;}
};

#endif
