/* 
 * Particle: A class representing a particle in space. 
 * Author: Ben P
 * Date: 04.08.04
 */

#ifndef PARTICLE_H
#define PARTICLE_H

#include "Vector3.h"

namespace Penguin
{ 
	/*! \brief A particle class.
	 *
	 * This class is an abstraction of a particle, the basic element of classical 
	 * mechanics. A particle has a mass, position, velocity, and acceleration in
	 * space but has an infinitisimal volume. Acceleration is a property that is
	 * dependant directly on Force and Mass, and thus we store a Force Accumulator
	 * instead of the current Acceleration.
	 */

	class Particle
	{
		protected:
		
		Vector3 m_pos;
		Vector3 m_vel;
		Vector3 m_force; /* force accumulator */
		double m_mass;
		
		public:
		
		Particle();
		Particle(Vector3& pos, Vector3& vel, double mass = 1.0);
		Particle(Vector3 pos, Vector3 vel = Vector3::ZERO, double mass = 1.0);
				
		/*! Returns the Position of this particle. */
		Vector3& GetPos();
		/*! Returns the Velocity of this particle. */
		Vector3& GetVel();
		/*! Returns the current force accumulator of this particle. */
		Vector3& GetForce();
		/*! Returns the mass. */
		double& GetMass();
		
		/*! Set Position */
		void SetPos(Vector3 pos);
		/*! Set Velocity */
		void SetVel(Vector3 vel);
		/*! Set Force Accumulator */
		void SetForce(Vector3 f);
		/*! Set Mass */
		void SetMass(double m);		
		
		/*! Abstract interface for all parametrically defined particles. */
		virtual void Simulate(double t){};
	};
}

#endif
