#ifndef VECTOR3_H
#define VECTOR3_H

#include "MiscMath.h"
#include <iostream>

namespace Penguin
{
	/*! \brief A 3d vector class.
 *
 * This class represents a vector in 3-space.
 */ 
	class Vector3
	{
		REAL m_data[3];
			
		public:
			
		Vector3();
		Vector3(REAL x, REAL y, REAL z);
		Vector3(const Vector3& v);
		
		/*! Accessor method */			
		REAL& operator[](int index);
		/*! Returns a pointer to the array of elements. */
		REAL* GetData(){ return m_data; }
		/*! A shortcut: return a particular member element. */
		REAL& GetX(){ return m_data[0]; }
		REAL& GetY(){ return m_data[1]; }
		REAL& GetZ(){ return m_data[2]; }		

/* Operations */
		Vector3& operator+=(const Vector3& v);
		Vector3& operator-=(const Vector3& v);				
		Vector3 operator+(const Vector3& v);
		Vector3 operator-(const Vector3& v);		
		Vector3 operator-(); //negation about all 3 axes		
		Vector3 operator/(REAL d);
		Vector3 operator*(REAL d);	
		Vector3& operator/=(REAL d);	
		bool operator==(const Vector3& v);
		bool operator!=(const Vector3& v);
		
		/* Procedures */
		/*! Returns the length (Euclidean norm). */
		REAL GetLength();
		/*! Returns the square length */
		REAL GetSqLength();
		/*! Returns the dot product of <b>this</b> and <b>v</b> */		
		REAL Dot(const Vector3& v) const;
		/*! Returns the cross product of <b>this</b> and <b>v</b> */
		Vector3 Cross(const Vector3& v) const;
		/*! Normalises the vector */
		Vector3& Normalise(); //normalises the vector
		/*! Returns a normalised version of the vector */
		Vector3 Normal() const;
		/*! Negation of all elements. */
		Vector3& Reverse();	//same as unary minus
		/*! ZERO = {0,0,0} */
		static const Vector3 ZERO;
		
		friend std::ostream& operator<<(std::ostream& out, Vector3& v);
		
		/*! Vector {1,0,0} */
		static Vector3 GetX_axis();
		/*! Vector {0,1,0} */
		static Vector3 GetY_axis();
		/*! Vector {0,0,1} */
		static Vector3 GetZ_axis();
		
	};
}

#endif
