#ifndef CLOTHY
#define CLOTHY

#include "GL/glut.h"
#include "PSystem.h"
#include "RigidStrut.h"
#include "MPWorld.h"
#include "Emitter.h"
#include <vector>

using namespace Penguin;

class BasicEmitter: public CollidableEmitter
{	
	public:
	
	BasicEmitter();
		
	void Draw();
};

class TestPSystem
{
	//Particle System info
	PSystem m_System;	
	BasicEmitter* m_Emitter;

	double w; //width/2 of cube sides
	
	//Width,Height of window
	int m_Width, m_Height;
	
	//Camera/Viewpoint Info
	Vector3 m_Camera;	
	Vector3 m_LookAt;
	
	/* frame data */
	int m_NumFrames;
	double m_FPS;
	int m_MSSinceLastDraw;
	
	/*! The milliseconds passed per redraw.*/
	int m_MSPerFrame;
		
	/*! The update flag. */
	bool m_IsUpdating;
	
	/*! Used to hack GLUT callbacks */
	static TestPSystem* m_Instance;
	
	/* prints a string */
	void PrintString(char* s);
	
	/* For MESA */
	void WindowPos2f( GLfloat x, GLfloat y);
	
	/* Internal Initialising Functions */	
	void InitialiseGlut();
	void InitialiseGL();
	
	public:
	TestPSystem();
	~TestPSystem(){}
	
	void Render();
	void Update();	

	void SetCamera(const Vector3& cam){m_Camera = cam;}
	void SetLookAt(const Vector3& lookat){m_LookAt = lookat;}	
	void Set3dViewport(double aspect);
	void Reshape(int w, int h);
	void Display(void);	
	void RenderGuides();	
	void EnableLighting();	
	void BeginMainLoop(void);
	void EnableTimer(int t);
	void DisplayFPS();
	int Time();
	
	void RotateAroundY(double angle);
	void LookUpDown(double angle);
	void LookLeftRight(double angle);
	void MoveUpDown(double distance);
	void MoveLeftRight(double distance);
	void ResetLookAt();
	void Zoom(double amount);
		
	void KeyListener(unsigned char key, int x, int y);
	void SpecialKeyListener(int key, int x, int y);
	void MouseClicked(int button, int state, int x, int y){};
	void MouseDragged(int x, int y){};
	void MouseMoved(int x, int y){};
			
	static void KEYBOARD_CALLBACK(unsigned char key, int x, int y){m_Instance->KeyListener(key,x,y);}	
	static void KEYBOARD_SPECIAL_CALLBACK(int key, int x, int y){m_Instance->SpecialKeyListener(key,x,y);}
	static void DISPLAY_CALLBACK(){m_Instance->Display();}	
	static void RESHAPE_CALLBACK(int w, int h){m_Instance->Reshape(w,h);}
	static void TIMER_CALLBACK(int t)
	{
		if (m_Instance->m_IsUpdating) m_Instance->Update();
		 
		m_Instance->m_NumFrames++;
		
		int dt = m_Instance->Time() - m_Instance->m_MSSinceLastDraw;
		
		if (dt>1000)
		{
			m_Instance->m_FPS = m_Instance->m_NumFrames*1000.0/dt;
			m_Instance->m_NumFrames = 0;
			m_Instance->m_MSSinceLastDraw = m_Instance->Time();
		}
						
		glutPostRedisplay();
		glutTimerFunc(m_Instance->m_MSPerFrame,&TestPSystem::TIMER_CALLBACK,0);
	}
	
	static void MOUSECLICK_CALLBACK(int button, int state, int x, int y)
	{
		m_Instance->MouseClicked(button,state,x,y);
	} 
	
	static void MOUSEMOVE_CALLBACK(int x, int y)
	{
		m_Instance->MouseDragged(x,y);
	}
	
	static void PASSIVEMOUSEMOVE_CALLBACK(int x, int y)
	{
		m_Instance->MouseMoved(x,y);
	}

	static void drawFace(ColOrientedFace *face);
	static void drawAABB(ColAABB *aabb, GLuint t);
};


#endif
