/* A simple test application class.
 * Subclass to get required functionality.
 * BP - 06.08.04
 */

#include "GL/glut.h"

#ifndef TESTBED_H
#define TESTBED_H

/*! \brief Defines a simple test application class.
 *
 * class TestBed provides the basic setup for an application using GLUT.
 * To use, simply subclass another class from TestBed and redefine the functions
 * you wish to change.
 *
 * Warning: Only one TestBed class should be instanced per application!
 * Otherwise the behaviour of GLUT will be undefined!
 *
 * Why? (Advanced)<br />
 * The way I implemented the C-Style callbacks is not very flexible. Feel free
 * to fix this.
 */ 

class TestBed
{
	public:

	/* window data */	
	int m_Width;
	int m_Height;
	
	/* viewport data */
	double m_Left;
	double m_Right;
	double m_Bottom;
	double m_Top;
	
	double m_CenterX;
	double m_CenterY;
	
	/* frame data */
	int m_NumFrames;
	double m_FPS;
	int m_MSSinceLastDraw;
	
	/*! The milliseconds passed per redraw.*/
	int m_MSPerFrame;
		
	/*! The update flag. */
	bool m_IsUpdating;
	
	/*! Used to hack GLUT callbacks */
	static TestBed* m_Instance;
	
	/* prints a string */
	void PrintString(char* s);
	
	/* For MESA */
	void WindowPos2f( GLfloat x, GLfloat y);
	
	
	public:
	
	TestBed(int argc, char** argv);
	TestBed(int argc, char** argv,int w, int h);
	virtual ~TestBed();	
	
	/*! You need to call this right after construction! */
	virtual void Setup();
	
	
	virtual void GlutSetup();
	virtual void GLSetup();
	
	void DisplayFPS();
	
	/*! This function centeres the viewpoint at x,y */
	void SetCenter(double x, double y);
	
	/*! Call this function to define the viewport */
	void SetViewport(double left, double right, double bottom, double top);	
	
	/*! Only redefine this function if you want more control over the drawing method. You probably only want to overload Render. */
	virtual void Display(void);		
	/*! Implement this function to send gl calls to the screen. */
	virtual void Render(void);
	/*! Redefine this function to handle key presses.
	*
	* key: is the the generated <b>ascii</b> code for they key being pressed
	* x,y: the coordinates of the mouse when the key was pressed
	*
	*/
	virtual void KeyListener(unsigned char key, int x, int y);
	/*! Inherit this to define mouse click response. 
	*
	* button: GLUT_LEFT_BUTTON, GLUT_MIDDLE_BUTTON, GLUT_RIGHT_BUTTON
	* state: GLUT_UP, GLUT_DOWN
	* 
	*/
	virtual void MouseClicked(int button, int state, int x, int y){};	
	/*! This callback is called whenever the mouse button is held down while the
	* mouse is being moved. */
	virtual void MouseDragged(int x, int y){};
	/*! This callback is called whenever the mouse is moved and the button has not been pressed. */
	virtual void MouseMoved(int x, int y){};	
	
	/*! Redefine this function to redefine the update method. */
	virtual void Update(void);
	/*! You shouldn't need to redefine this. */
	virtual void Reshape(int w, int h);
		
	/*! Call this function to send the application into its event processing loop */
	void BeginMainLoop(void);
	
	/*! Call this function to enable the timing callback. */
	void EnableTimer(int t);
	
	/*! Returns the time in milliseconds since the App began. */
	int Time();
	
		
	
	
	
	
		
	/* Some callback hacks. */	
	static void KEYBOARD_CALLBACK(unsigned char key, int x, int y){m_Instance->KeyListener(key,x,y);}	
	static void DISPLAY_CALLBACK(){m_Instance->Display();}	
	static void RESHAPE_CALLBACK(int w, int h){m_Instance->Reshape(w,h);}
	static void TIMER_CALLBACK(int t)
	{
		if (m_Instance->m_IsUpdating) m_Instance->Update();
		 
		m_Instance->m_NumFrames++;
		
		int dt = m_Instance->Time() - m_Instance->m_MSSinceLastDraw;
		
		if (dt>1000)
		{
			m_Instance->m_FPS = m_Instance->m_NumFrames*1000.0/dt;
			m_Instance->m_NumFrames = 0;
			m_Instance->m_MSSinceLastDraw = m_Instance->Time();
		}
						
		glutPostRedisplay();
		glutTimerFunc(m_Instance->m_MSPerFrame,&TestBed::TIMER_CALLBACK,0);
	}
	
	static void MOUSECLICK_CALLBACK(int button, int state, int x, int y)
	{
		m_Instance->MouseClicked(button,state,x,y);
	} 
	
	static void MOUSEMOVE_CALLBACK(int x, int y)
	{
		m_Instance->MouseDragged(x,y);
	}
	
	static void PASSIVEMOUSEMOVE_CALLBACK(int x, int y)
	{
		m_Instance->MouseMoved(x,y);
	}
	
};

#endif
