#include "clothy2.h"
#include "IL/ilut.h"

#include <iostream>

using namespace Penguin;

double speed_factor = 1; //i.e. how much should we update the world relative to the screen

TestPSystem* TestPSystem::m_Instance = NULL;

GLuint texname;

int main(int argc, char** argv)
{
	glutInit(&argc,argv);
	
	ilInit();
	iluInit();
	ilutRenderer(ILUT_OPENGL);	

	TestPSystem test;
	test.EnableTimer(5);
	test.BeginMainLoop();
	
	return 0;
}

TestPSystem::TestPSystem():w(2),m_Width(500),m_Height(500), m_NumFrames(0), m_FPS(0), m_MSSinceLastDraw(0), m_MSPerFrame(0), m_IsUpdating(true)
{
	InitialiseGlut();
	InitialiseGL();

	World* world = m_System.AddWorld("Test", new World(5,0.02));
	if (!world)
	{
		std::cout << "Can't create world!" << std::endl;
	}
	else
	{	
		int width = 10;
	
		/* generate a new patch */
		m_Cloth = new MyClothPatch(width,3,2000,100,2000,0.6);	
		
		//pin the corners up//
		
		Object* pins = new Object();
		
		StableParticle* p1 = new StableParticle(Vector3(-1.5,0.0005,-1.5));
		StableParticle* p2 = new StableParticle(Vector3(-1.5,0.0005,1.5));
		StableParticle* p3 = new StableParticle(Vector3(1.5,0.0005,1.5));
		StableParticle* p4 = new StableParticle(Vector3(1.5,0.0005,-1.5));
		
		pins->AddParametricParticle(p1);
		pins->AddParametricParticle(p2);
		pins->AddParametricParticle(p3);
		pins->AddParametricParticle(p4);
		
		double l = 0.0005;
		double k = 10000;
		double c = 10;
		
		DampedSpring* dp1 = new DampedSpring(p3,m_Cloth->GetParticle(0,width-1),l,k,c);
		DampedSpring* dp2 = new DampedSpring(p4,m_Cloth->GetParticle(width-1,width-1),l,k,c);
		DampedSpring* dp3 = new DampedSpring(p1,m_Cloth->GetParticle(width-1,0),l,k,c);
		DampedSpring* dp4 = new DampedSpring(p2,m_Cloth->GetParticle(0,0),l,k,c);
		
		pins->AddForce(dp1);		
		pins->AddForce(dp2);
		pins->AddForce(dp3);
		pins->AddForce(dp4);
		
		Object* ball = new Object();
		ball->SetInternalCollisions(false);
		
		/* add a collidable ball */
		
		Particle* sp = new Particle(Vector3(0,5,0));
		sp->SetMass(200);
		
		m_b = new ColSphere(sp, 0.7);
		m_b->SetCoefficientOfRestitution(0.55);
		
		ball->AddParticle(sp);
		ball->AddCollisionStruct(m_b);
				
		ColPlane* cp = new ColPlane(Vector3(0,1,0),Vector3(0,-w,0));		
		ball->AddCollisionStruct(cp);
		
		cp = new ColPlane(Vector3(0,-1,0),Vector3(0,w,0));		
		ball->AddCollisionStruct(cp); // ceiling	
		cp = new ColPlane(Vector3(1,0,0),Vector3(-w,0,0));		
		ball->AddCollisionStruct(cp);	// v wall
		cp = new ColPlane(Vector3(-1,0,0),Vector3(w,0,0));		
		ball->AddCollisionStruct(cp);	
		cp = new ColPlane(Vector3(0,0,1),Vector3(0,0,-w));		
		ball->AddCollisionStruct(cp);	
		cp = new ColPlane(Vector3(0,0,-1),Vector3(0,0,w));		
		ball->AddCollisionStruct(cp);		
				// Mikes first contribution !
				
		/* finally add the cloth patch to the world. */
		world->ManageObject(m_Cloth,"cloth");	
		world->ManageObject(ball,"ball");	
		world->ManageObject(pins,"pins");		
				
		m_Cloth->NextDrawMode();
		m_Cloth->NextDrawMode();
		m_Cloth->NextDrawMode();
		
		SetCamera(Vector3(1.75,2.3,-3.5));
		SetLookAt(Vector3(0,0,0));
		
		EnableLighting();
	}	
}

void TestPSystem::Render(void)
{
	glPointSize(5);
	m_Cloth->Draw();
	
	Particle* p = m_b->GetTarget();
	
	glColor3f(0.5,0.5,1);
	glTranslatef(p->GetPos()[0],p->GetPos()[1],p->GetPos()[2]);
	glutSolidSphere(m_b->GetRadius()-0.05,32,32);
	glTranslatef(-p->GetPos()[0],-p->GetPos()[1],-p->GetPos()[2]);
	
	glColor3f(1,1,1);
	
	glDisable(GL_LIGHTING);
	glutWireCube(w*2);
	glEnable(GL_LIGHTING);	
}

void TestPSystem::Update(void)
{
	static double dt = m_MSPerFrame/1000.0;
	m_System.Simulate(dt*speed_factor);
}

void TestPSystem::KeyListener(unsigned char key, int x, int y)
{
	if (key==' ') m_IsUpdating = !m_IsUpdating;	
	if (key=='a') RotateAroundY(0.1);	
	else if (key=='d') RotateAroundY(-0.1);	
	else if (key=='w') Zoom(1.1);
	else if (key=='s') Zoom(0.9);
	else if (key=='\t') m_Cloth->NextDrawMode();
}

/* SET UP GLUT AND OPENGL */
void TestPSystem::InitialiseGlut()
{
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

	//Create a single window
	glutInitWindowPosition(100,100);
	glutInitWindowSize(m_Width,m_Height);		
	glutCreateWindow("Cloth Test");		
	
	//Callbacks
	glutDisplayFunc(&TestPSystem::DISPLAY_CALLBACK);
	glutReshapeFunc(&TestPSystem::RESHAPE_CALLBACK);
	glutKeyboardFunc(&TestPSystem::KEYBOARD_CALLBACK);	
	glutMouseFunc(&TestPSystem::MOUSECLICK_CALLBACK);
	glutMotionFunc(&TestPSystem::MOUSEMOVE_CALLBACK);
	glutPassiveMotionFunc(&TestPSystem::PASSIVEMOUSEMOVE_CALLBACK);
	
	glutSetCursor(GLUT_CURSOR_CROSSHAIR);
}

void TestPSystem::InitialiseGL()
{
	glClearColor(0,0,0,0);	
	double aspect = (double)m_Height/m_Width;	
	Set3dViewport(aspect);
	
	
	texname = ilutGLLoadImage("cloth.png");
}

void TestPSystem::Reshape(int w, int h)
{
	m_Width = w;
	m_Height = h;
	
	double aspect = (double)m_Height/m_Width;
		
	glViewport(0,0,w,h);
	
	Set3dViewport(aspect);
}

void TestPSystem::Display(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	
	bool resetLight = false;
	
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	
	DisplayFPS();	
	
	glEnable(GL_DEPTH_TEST);	
	glEnable(GL_LIGHTING);
		gluLookAt(m_Camera[0],m_Camera[1],m_Camera[2],m_LookAt[0],m_LookAt[1],m_LookAt[2],0,1,0);
	
	Render();
	
	glutSwapBuffers();
}

void TestPSystem::Set3dViewport(double aspect)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60,1/aspect,.1,1000); 
	
	glMatrixMode(GL_MODELVIEW);
}

void TestPSystem::RotateAroundY(double angle)
{
	Vector3 dV = m_Camera - m_LookAt;
	
	double x = dV[0];
	double z = dV[2];
	
	x = dV[0]*cos(angle) + dV[2]*sin(angle);
	z = -dV[0]*sin(angle) + dV[2]*cos(angle);
	
	dV[0] = x;
	dV[2] = z;
	
	m_Camera = dV + m_LookAt;	
}

void TestPSystem::RenderGuides()
{
	glColor3f(0.5,0.5,0.5);
	
	glBegin(GL_LINE_LOOP);
		glVertex3f(-1,-1,-1);
		glVertex3f(1,-1,-1);
		glVertex3f(1,-1,1);
		glVertex3f(-1,-1,1);
	glEnd();
	
	glBegin(GL_LINE_LOOP);
		glVertex3f(-1,1,-1);
		glVertex3f(1,1,-1);
		glVertex3f(1,1,1);
		glVertex3f(-1,1,1);
	glEnd();
	
	glBegin(GL_LINES);
		glVertex3f(-1,-1,-1);
		glVertex3f(-1,1,-1);
		
		glVertex3f(1,-1,-1);
		glVertex3f(1,1,-1);
		
		glVertex3f(1,-1,1);
		glVertex3f(1,1,1);
		
		glVertex3f(-1,-1,1);
		glVertex3f(-1,1,1);		
	glEnd();

}

void TestPSystem::EnableLighting()
{

	GLfloat col[] = {1,1,1,1.0};
	
	GLfloat spec[] = {.1,.1,.1,1.0};
	GLfloat shine[] = {1.0};
	
	GLfloat pos[] = {-0.1,5,10,1};
	
	GLfloat diff[] = {1.0,1.0,1.0,1.0};

	GLfloat lmodel_ambient[] = {0.2,0.2,0.2,1};
	
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, col);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, spec);	
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shine);	
		
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diff);
	glLightfv(GL_LIGHT0, GL_SPECULAR, diff);
	glLightfv(GL_LIGHT0, GL_POSITION, pos);
		
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
	
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_COLOR_MATERIAL);
}

void TestPSystem::Zoom(double amount)
{
	Vector3 dV = m_Camera - m_LookAt;
	dV = dV/amount;
	m_Camera = m_LookAt + dV;	
}

void TestPSystem::BeginMainLoop(void)
{
	TestPSystem::m_Instance = this;
	glutMainLoop();
}

void TestPSystem::EnableTimer(int t)
{
	m_MSPerFrame = t;
	glutTimerFunc(t,&TestPSystem::TIMER_CALLBACK,0);
}

int TestPSystem::Time()
{
	return glutGet(GLUT_ELAPSED_TIME);
}

void TestPSystem::PrintString(char* s)
{
	static void* font = {GLUT_BITMAP_9_BY_15};

	if (s && std::strlen(s))
	{
		while (*s)
		{
			glutBitmapCharacter((void*)font,*s);
			s++;
		}
	}
}

void TestPSystem::DisplayFPS()
{
	glColor3f(1,1,1);
	WindowPos2f(m_Width-100,m_Height-15);

	char buffer[10];
	std::sprintf(buffer,"fps: %g",m_FPS);	
	PrintString(buffer);
	
	WindowPos2f(m_Width-109,m_Height-30);
	double mspf = 1000*1/m_FPS; //milliseconds per frame
	std::sprintf(buffer,"mspf: %g",mspf);
	PrintString(buffer);
		
	WindowPos2f(m_Width-109,m_Height-45);
	std::sprintf(buffer,"msps: %g",mspf- m_MSPerFrame);
	PrintString(buffer);
}

void TestPSystem::WindowPos2f( GLfloat x, GLfloat y)
{	
	double z = 0;
	double w = 1;

   GLfloat fx, fy;

   /* Push current matrix mode and viewport attributes */
   glPushAttrib( GL_TRANSFORM_BIT | GL_VIEWPORT_BIT );

   /* Setup projection parameters */
   glMatrixMode( GL_PROJECTION );
   glPushMatrix();
   glLoadIdentity();
   glMatrixMode( GL_MODELVIEW );
   glPushMatrix();
   glLoadIdentity();

   glDepthRange( z, z );
   glViewport( (int) x - 1, (int) y - 1, 2, 2 );

   /* set the raster (window) position */
   fx = x - (int) x;
   fy = y - (int) y;
   glRasterPos4f( fx, fy, 0.0, w );

   /* restore matrices, viewport and matrix mode */
   glPopMatrix();
   glMatrixMode( GL_PROJECTION );
   glPopMatrix();

   glPopAttrib();
}

/*****************************************CLOTHPATCH****************************/
/*****************************************CLOTHPATCH****************************/
/*****************************************CLOTHPATCH****************************/

MyClothPatch::MyClothPatch(): m_Width(10), m_CurrentDrawMode(0)
{
	SetupPatch(1.0,1.0,1.0,1.0,0.5);
}

MyClothPatch::MyClothPatch(int m, double width, double structuralStrength, double shearStrength, double bend,double mass): m_Width(m), m_CurrentDrawMode(0)
{
	SetupPatch(width,structuralStrength,shearStrength, bend,mass);
}

void MyClothPatch::SetupPatch(double width, double str, double shear, double bend, double mass)
{
	/* Create a Cloth Patch */
	
		double strc = str/80;
		double shearc = shear/80;
		double bendc = bend/80;		
		
		double dist = width/m_Width;		
				
		int i,j;
		for (i=0; i<m_Width; i++) //row
		{
			for (j=0; j<m_Width; j++) //col
			{
				double x = width*((double)j/m_Width-0.5);
				double y = width*((1-(double)i/m_Width)-0.5);
			
				Particle* p = new Particle();			
				p->SetPos(Vector3(x,0,y));
				p->SetMass(mass);

				AddParticle(p);      				
				
				ColSphere* cs = new ColSphere(p,0.005);
				cs->SetCoefficientOfRestitution(0);
				AddCollisionStruct(cs);
			}
		}

		for (i=0; i<m_Width-1; i++) //row
		{
			for (j=0; j<m_Width-1; j++) //col
			{
				int row = i*m_Width+j;
			
				MyTri one;
				one.p1 = m_Particles[row];
				one.p2 = m_Particles[row+m_Width]; //next row
				one.p3 = m_Particles[row+1];
			
				m_Tris.push_back(one);
				
				MyTri two;
				two.p1 = m_Particles[row+m_Width];
				two.p2 = m_Particles[row+m_Width+1];
				two.p3 = m_Particles[row+1];
				
				m_Tris.push_back(two);
			}
		}
					
		/* horizontal springs */
		
		for(i=0; i<(m_Width); i++)
		{
			for(j=0; j<(m_Width-1); j++)
			{
				m_StructuralSprings.push_back(new DampedSpring( m_Particles[i*m_Width+j],
				m_Particles[i*m_Width+j+1],dist,str,strc));
	
				if (j<(m_Width-2))
				{
					m_BendSprings.push_back(new DampedSpring( m_Particles[i*m_Width+j],
					m_Particles[i*m_Width+j+2],2*dist,bend,bendc));
				}
			}
		}
		
		/* vertical springs */
		
		for(i=0; i<(m_Width-1); i++)
		{
			for(j=0; j<(m_Width); j++)
			{
				m_StructuralSprings.push_back(new DampedSpring(m_Particles[i*m_Width+j],m_Particles[(i+1)*m_Width+j],dist,str,strc));
				
				if (i<(m_Width-2))
				{
					m_BendSprings.push_back(new DampedSpring( m_Particles[i*m_Width+j],
					m_Particles[(i+2)*m_Width+j],2*dist,bend,bendc));
				}
			}
		}	
		
		/* diagonal springs */
		
		double diag = sqrt(dist*dist);
		
		for(i=0; i<(m_Width-1); i++)
		{
			for(j=0; j<(m_Width-1); j++)
			{
				m_ShearSprings.push_back(new DampedSpring(m_Particles[i*m_Width+j],m_Particles[(i+1)*m_Width+j+1],diag,shear,shearc));
				
				m_ShearSprings.push_back(new DampedSpring(m_Particles[(i+1)*m_Width+j],m_Particles[i*m_Width+j+1],diag,shear,shearc));			
			}
		}
}

MyClothPatch::~MyClothPatch()
{
	std::vector<Particle*>::iterator pit = m_Particles.begin();
	while(pit!=m_Particles.end())
	{
		delete(*pit);
		pit++;
	}
	
	std::vector<Penguin::DampedSpring*>::iterator it = m_StructuralSprings.begin();
	while(it!=m_StructuralSprings.end())
	{
		delete(*it);
		it++;
	}
	
	it = m_ShearSprings.begin();
	while(it!=m_ShearSprings.end())
	{
		delete(*it);
		it++;
	}
	
	it = m_BendSprings.begin();
	while(it!=m_BendSprings.end())
	{
		delete(*it);
		it++;
	}
}

void MyClothPatch::Draw()
{
	switch (m_CurrentDrawMode)
	{
		case 0:
		{
			glPointSize(1);
			glColor3f(1,1,1);
			
			glDisable(GL_LIGHTING);
			DrawPoints();
			glEnable(GL_LIGHTING);
			
			break;
		}
		
		case 1:
		{
			glDisable(GL_LIGHTING);
			glPointSize(3);
			glColor3f(1,1,1);
			DrawPoints();
			
			glColor3f(0.3,0.3,0.6);
			DrawStructs();
			glEnable(GL_LIGHTING);
			break;
		}
		
		case 2:
		{
			glDisable(GL_LIGHTING);
			glPointSize(3);
			glColor3f(1,1,1);
			DrawPoints();
			
			glColor3f(0.3,0.3,0.6);
			DrawStructs();
			
			glColor3f(0.6,0.0,0.0);
			DrawShears();
			glEnable(GL_LIGHTING);
			break;			
		}
		
		case 3:
		{
			glColor3f(1,1,0);
			DrawPolygons();
			break;
		}
	}
	
}

Particle*& MyClothPatch::GetParticle(int i, int j)
{
	if (i<0||i>=m_Width||j<0||j>=m_Width)
	{
		return m_Particles[0];
	}
	
	return m_Particles[i*m_Width+j];
}

void MyClothPatch::ApplyForces()
{
	std::vector<DampedSpring*>::iterator it = m_StructuralSprings.begin();
	
	for(;it!=m_StructuralSprings.end();++it)
	{
		(*it)->ApplyForce();
	}
	
	it = m_ShearSprings.begin();
	
	for(;it!=m_ShearSprings.end();++it)
	{
		(*it)->ApplyForce();
	}
	
	it = m_BendSprings.begin();
	
	for(;it!=m_BendSprings.end();++it)
	{
		(*it)->ApplyForce();
	}
}

void MyClothPatch::NextDrawMode()
{
	m_CurrentDrawMode++;
	if (m_CurrentDrawMode>3) m_CurrentDrawMode = 0;
	
}

void MyClothPatch::DrawStructs()
{
	glBegin(GL_LINES);
	
	std::vector<Penguin::DampedSpring*>::iterator sMyTrit = m_StructuralSprings.begin();
	while(sMyTrit!=m_StructuralSprings.end())
	{
		glVertex3dv((*sMyTrit)->GetOne()->GetPos().GetData());
		glVertex3dv((*sMyTrit)->GetTwo()->GetPos().GetData());
		sMyTrit++;
	}
	
	glEnd();
}

void MyClothPatch::DrawShears()
{
	glBegin(GL_LINES);
	
	std::vector<Penguin::DampedSpring*>::iterator sMyTrit = m_ShearSprings.begin();
	while(sMyTrit!=m_ShearSprings.end())
	{
		glVertex3dv((*sMyTrit)->GetOne()->GetPos().GetData());
		glVertex3dv((*sMyTrit)->GetTwo()->GetPos().GetData());
		sMyTrit++;
	}
	
	glEnd();
}

void MyClothPatch::DrawPoints()
{
	glBegin(GL_POINTS);

	PIter pit = m_Particles.begin();
	while(pit!=m_Particles.end())
	{
		glVertex3dv((*pit)->GetPos().GetData());
		pit++;
	}	
	
	glEnd();
}

void MyClothPatch::DrawPolygons()
{
	std::vector<MyTri>::iterator it = m_Tris.begin();
	
	glColor3f(1,1,0);
	
	std::map<Particle*,Vector3> normalData;
	
	int i,j;
	
	for(i=0;i<m_Width;++i) //row
	{
		for(j=0;j<m_Width;++j) //col
		{
			int pos = i*m_Width+j; //pos in array
		
			Vector3 norm;
			
			if (i==0&&j==0)
			{
				norm = (p(pos+1)-p(pos)).Cross(p(pos+m_Width)-p(pos));
				norm.Normalise();
			}
			else if (i==0&&j==(m_Width-1))
			{	
				norm = (p(pos-1)-p(pos)).Cross(p(pos+m_Width)-p(pos));
				norm.Normalise();
			}
			else if (i==0)
			{
				Vector3 a = p(pos+m_Width);
				Vector3 b = p(pos+1);
				Vector3 c = p(pos-1);
				Vector3 x = p(pos);
				
				norm = (a-x).Cross(b-x) + (c-x).Cross(a-x);
				norm.Normalise();
			}
			else if (i==(m_Width-1)&&j==0)
			{
				norm = (p(pos+1)-p(pos)).Cross(p(pos-m_Width)-p(pos));
				norm.Normalise();
			}
			else if (i==(m_Width-1)&&j==(m_Width-1))
			{	
				norm = -(p(pos-1)-p(pos)).Cross(p(pos-m_Width)-p(pos));
				norm.Normalise();
			}
			else if (i==(m_Width-1))
			{
				Vector3 a = p(pos-m_Width);
				Vector3 b = p(pos+1);
				Vector3 c = p(pos-1);
				Vector3 x = p(pos);
				
				norm = (a-x).Cross(c-x) + (b-x).Cross(a-x);
				norm.Normalise();
			}
			else if (j==0)
			{
				Vector3 a = p(pos-m_Width);
				Vector3 b = p(pos+m_Width);
				Vector3 c = p(pos+1);
				Vector3 x = p(pos);
				
				norm = (c-x).Cross(a-x) + (b-x).Cross(c-x);
				norm.Normalise();
			}
			else if (j==(m_Width-1))
			{
				Vector3 a = p(pos-m_Width);
				Vector3 b = p(pos+m_Width);
				Vector3 c = p(pos-1);
				Vector3 x = p(pos);
				
				norm = (a-x).Cross(c-x) + (c-x).Cross(b-x);
				norm.Normalise();
			}
			else
			{
				Vector3 a = p(pos-1);
				Vector3 b = p(pos+m_Width);
				Vector3 c = p(pos+1);
				Vector3 d = p(pos-m_Width);
				Vector3 x = p(pos);
				
			//	norm = (a-c).Cross(b-d);
			
				norm =	(b-x).Cross(a-x) + 
								(a-x).Cross(d-x) + 
								(d-x).Cross(c-x) + 
								(c-x).Cross(b-x);
									
				norm.Normalise();
				norm.Reverse();
			}			
								
			normalData[m_Particles[pos]] = norm;			
						
		} //end col
	}	//end row
	
	
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D,texname);
	
	glBegin(GL_TRIANGLES);
	
	int x=0,y=0;
	double sqW = 1/(double)(m_Width-1);
		
	for(;it!=m_Tris.end();++it)
	{
		/*** UPPER LEFT TRIANGLE ***/
	
		Particle* one = it->p1;		
		glNormal3dv(normalData[one].GetData());	
		glTexCoord2d(x*sqW,y*sqW);
		glVertex3dv(one->GetPos().GetData());
		
		Particle* two = it->p2;		
		glNormal3dv(normalData[two].GetData());	
		glTexCoord2d(x*sqW,(y+1)*sqW);
		glVertex3dv(two->GetPos().GetData());
		
		Particle* thr = it->p3;		
		glNormal3dv(normalData[thr].GetData());	
		glTexCoord2d((x+1)*sqW,y*sqW);		
		glVertex3dv(thr->GetPos().GetData());
		
		++it;
		/** LOWER RIGHT TRIANGLE ***/
	
		one = it->p1;		
		glNormal3dv(normalData[one].GetData());	
		glTexCoord2d(x*sqW,(y+1)*sqW);				
		glVertex3dv(one->GetPos().GetData());
		
		two = it->p2;		
		glNormal3dv(normalData[two].GetData());	
		glTexCoord2d((x+1)*sqW,(y+1)*sqW);		
		glVertex3dv(two->GetPos().GetData());
		
		thr = it->p3;		
		glNormal3dv(normalData[thr].GetData());	
		glTexCoord2d((x+1)*sqW,y*sqW);		
		glVertex3dv(thr->GetPos().GetData());
		
		x++;
		if (x==m_Width)
		{
			x = 0;
			y++;
		}
	}	
	glEnd();
	
	glDisable(GL_TEXTURE_2D);
	
}




