#include "emittest.h"
#include "IL/ilut.h"

#include <iostream>

using namespace Penguin;

double speed_factor = 1; //i.e. how much should we update the world relative to the screen

TestPSystem* TestPSystem::m_Instance = NULL;

ColAABB* chimney;
ColAABB* hedge1;
ColOrientedFace* roof1;
ColOrientedFace* roof2;
ColOrientedFace* roof3;
/*
ColOrientedFace* wall1;
ColOrientedFace* wall2;
ColOrientedFace* wall3;
ColOrientedFace* wall4;
ColOrientedFace* wall5;*/
ColAABB* wall1;
ColAABB* wall2;
ColAABB* wall3;
ColAABB* wall4;
ColAABB* wall5;


GLuint floortex;
GLuint rooftex;
GLuint walltex;
GLuint hedgetex;

int main(int argc, char** argv)
{
	glutInit(&argc,argv);
	
	ilInit();
	iluInit();
	ilutRenderer(ILUT_OPENGL);	

	TestPSystem test;
	test.EnableTimer(5);
	test.BeginMainLoop();	
	return 0;
}

TestPSystem::TestPSystem():w(2),m_Width(500),m_Height(500), m_NumFrames(0), m_FPS(0), m_MSSinceLastDraw(0), m_MSPerFrame(0), m_IsUpdating(true)
{
	InitialiseGlut();
	InitialiseGL();

	World* world = m_System.AddWorld("Test", new World(5,0.02));
	if (!world)
	{
		std::cout << "Can't create world!" << std::endl;
	}
	else
	{
		ColPlane* pCP = new ColPlane(Vector3(0,1,0),Vector3(0,-w/2,0)); 
		Object* po = new Object();
		po->AddCollisionStruct(pCP);
		
		chimney = new ColAABB(Vector3(1,0,0),0.5,2,0.5);
		hedge1 = new ColAABB(Vector3(3.5,-0.75,0),0.5,0.5,7);
		roof1 = new ColOrientedFace(Vector3(1,0,0),Vector3(1,1,0),Vector3(-1,1,0),4,1.5);
		roof2 = new ColOrientedFace(Vector3(-2,0,0),Vector3(-1,1,0),Vector3(1,1,0),4,1.5);
		roof3 = new ColOrientedFace(Vector3(-0.5,0.5,0),Vector3(0,1,0),Vector3(1,0,0),4,2);
	
		wall1 = new ColAABB(Vector3(1,-0.5,0),0.25,1,4);
		wall2 = new ColAABB(Vector3(-2,-0.5,0),0.25,1,4);
		wall3 = new ColAABB(Vector3(-0.5,-0.5,2),3,1,0.25);
		wall4 = new ColAABB(Vector3(0.5,-0.5,-2),1,1,0.25);
		wall5 = new ColAABB(Vector3(-1.5,-0.5,-2),1,1,0.25);
	
		po->AddCollisionStruct(chimney);
		po->AddCollisionStruct(hedge1);
		po->AddCollisionStruct(roof1);
		po->AddCollisionStruct(roof2);
		po->AddCollisionStruct(roof3);
		po->AddCollisionStruct(wall1);
		po->AddCollisionStruct(wall2);
		po->AddCollisionStruct(wall3);
		po->AddCollisionStruct(wall4);
		po->AddCollisionStruct(wall5);
				

		world->ManageObject(po,"Other");

		m_Emitter = (BasicEmitter*) world->ManageObject(new BasicEmitter(), "Emitter");
		m_Emitter->SetLife(5);
		m_Emitter->SetRate(0.005);
	  m_Emitter->SetBehaviour(new PlanarEmitter(Vector3(0,3,0),4*w,4*w));	
		m_Emitter->SetInternalCollisions(false);

		SetCamera(Vector3(1.75,2.3,-3.5));
		SetLookAt(Vector3(0,0,0));
		EnableLighting();
	}	
}
	
void TestPSystem::Render(void)
{
	glEnable(GL_TEXTURE_2D);
	glPointSize(5);

	/* Draw the chimney */
	drawAABB(chimney, walltex);

	/*Draw the hedge */
	drawAABB(hedge1, hedgetex);

	/* Draw the objects */

	glBindTexture(GL_TEXTURE_2D,rooftex);
	/* Draw the Roof */
	drawFace(roof1);
	drawFace(roof2);
	drawFace(roof3);

	glBindTexture(GL_TEXTURE_2D,walltex);
	/* Draw the wall */
	/*drawFace(wall1);
	drawFace(wall2);
	drawFace(wall3);
	drawFace(wall4);
	drawFace(wall5);*/
	drawAABB(wall1, walltex);
	drawAABB(wall2, walltex);
	drawAABB(wall3, walltex);
	drawAABB(wall4, walltex);
	drawAABB(wall5, walltex);

	glBindTexture(GL_TEXTURE_2D,floortex);

	/* Draw the ground */	
	REAL w2 = w/2;
	REAL d /*detail*/ = 40;

	glBegin(GL_QUADS);
		glNormal3f(0,1,0);

		glTexCoord2f(0,0);
		glVertex3f(-4*w,-w2,-4*w);
	
		glTexCoord2f(d,0);
		glVertex3f(4*w,-w2,-4*w);

		glTexCoord2f(d,d);
		glVertex3f(4*w,-w2,4*w);

		glTexCoord2f(0,d);
		glVertex3f(-4*w,-w2,4*w);
	glEnd();

	glDisable(GL_TEXTURE_2D);

	m_Emitter->Draw();
}

void TestPSystem::Update(void)
{
	static double dt = m_MSPerFrame/1000.0;
	m_System.Simulate(dt*speed_factor);
}

void TestPSystem::KeyListener(unsigned char key, int x, int y)
{
	if (key==' ') m_IsUpdating = !m_IsUpdating;	
	if (key=='d') MoveLeftRight(0.1);	
	else if (key=='a') MoveLeftRight(-0.1);	
	else if (key=='w') Zoom(0.1);
	else if (key=='s') Zoom(-0.1);
	else if (key=='x') ResetLookAt();
	else if (key=='r') MoveUpDown(0.1);
	else if (key=='f') MoveUpDown(-0.1);
}

void TestPSystem::SpecialKeyListener(int key, int x, int y)
{
	if (key==GLUT_KEY_UP) LookUpDown(0.1);
	else if (key==GLUT_KEY_DOWN) LookUpDown(-0.1);
	else if (glutGetModifiers()==GLUT_ACTIVE_CTRL && key==GLUT_KEY_LEFT)
	{RotateAroundY(0.1);}
	else if (glutGetModifiers()==GLUT_ACTIVE_CTRL && key==GLUT_KEY_RIGHT)
	{RotateAroundY(-0.1);}
	else if (key==GLUT_KEY_LEFT) LookLeftRight(0.1);
	else if (key==GLUT_KEY_RIGHT) LookLeftRight(-0.1);
}
/* SET UP GLUT AND OPENGL */
void TestPSystem::InitialiseGlut()
{
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

	//Create a single window
	glutInitWindowPosition(100,100);
	glutInitWindowSize(m_Width,m_Height);		
	glutCreateWindow("Emitter Test");		
	
	//Callbacks
	glutDisplayFunc(&TestPSystem::DISPLAY_CALLBACK);
	glutReshapeFunc(&TestPSystem::RESHAPE_CALLBACK);
	glutKeyboardFunc(&TestPSystem::KEYBOARD_CALLBACK);	
	glutSpecialFunc(&TestPSystem::KEYBOARD_SPECIAL_CALLBACK);
	glutMouseFunc(&TestPSystem::MOUSECLICK_CALLBACK);
	glutMotionFunc(&TestPSystem::MOUSEMOVE_CALLBACK);
	glutPassiveMotionFunc(&TestPSystem::PASSIVEMOUSEMOVE_CALLBACK);
	
	glutSetCursor(GLUT_CURSOR_CROSSHAIR);
}

void TestPSystem::InitialiseGL()
{
	glClearColor(0,0,0.8,0);	
	double aspect = (double)m_Height/m_Width;	
	Set3dViewport(aspect);
	
	floortex = ilutGLLoadImage("floor.png");
	rooftex = ilutGLLoadImage("roof.png");
	walltex = ilutGLLoadImage("wall.png");
	hedgetex = ilutGLLoadImage("hedge.png");
}

void TestPSystem::Reshape(int w, int h)
{
	m_Width = w;
	m_Height = h;
	
	double aspect = (double)m_Height/m_Width;
		
	glViewport(0,0,w,h);
	
	Set3dViewport(aspect);
}

void TestPSystem::Display(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	bool resetLight = false;
	
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	
	DisplayFPS();	
	
	glEnable(GL_DEPTH_TEST);	
	glEnable(GL_LIGHTING);
	gluLookAt(m_Camera[0],m_Camera[1],m_Camera[2],m_LookAt[0],m_LookAt[1],m_LookAt[2],0,1,0);
	
	Render();
	
	glutSwapBuffers();
}

void TestPSystem::Set3dViewport(double aspect)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60,1/aspect,.1,1000); 
	
	glMatrixMode(GL_MODELVIEW);
}

void TestPSystem::RotateAroundY(double angle)
{
	Vector3 dV = m_Camera - m_LookAt;
	
	double x = dV[0];
	double z = dV[2];
	
	x = dV[0]*cos(angle) + dV[2]*sin(angle);
	z = -dV[0]*sin(angle) + dV[2]*cos(angle);
	
	dV[0] = x;
	dV[2] = z;
	
	m_Camera = dV + m_LookAt;	
}

void TestPSystem::LookUpDown(double angle)
{
	Vector3 dV = m_LookAt - m_Camera;
	REAL d = tan(angle) * dV.GetLength();
	Vector3 temp = m_LookAt + Vector3(0, d, 0);
	m_LookAt = m_Camera + (temp - m_Camera).Normal()*dV.GetLength();
}

void TestPSystem::LookLeftRight(double angle)
{
	
	Vector3 dV = m_LookAt - m_Camera;
	
	double x = dV[0];
	double z = dV[2];
	
	x = dV[0]*cos(angle) + dV[2]*sin(angle);
	z = -dV[0]*sin(angle) + dV[2]*cos(angle);
	
	dV[0] = x;
	dV[2] = z;
	
	m_LookAt = dV + m_Camera;	
}

void TestPSystem::MoveUpDown(double distance)
{
	m_Camera += Vector3(0, distance, 0);
	m_LookAt += Vector3(0, distance, 0);
}

void TestPSystem::MoveLeftRight(double distance)
{
	Vector3 dV = m_Camera - m_LookAt;
	REAL old_distance = dV.GetSqLength();
	REAL new_distance = sqrt(old_distance + (distance*distance));
	REAL cos_a = dV.GetLength()/new_distance;	
	REAL sine_a = distance/new_distance;

	dV[0] = dV[0]*cos_a + dV[2]*sine_a;
	dV[2] = -dV[0]*sine_a + dV[2]*cos_a;

	Vector3 new_Camera = m_LookAt + dV.Normal()*new_distance;
	m_LookAt = m_LookAt + (new_Camera - m_Camera);
	m_Camera = new_Camera;	
}

void TestPSystem::ResetLookAt()
{
	m_LookAt = m_LookAt*0;
	
}

void TestPSystem::RenderGuides()
{
	glColor3f(0.5,0.5,0.5);
	
	glBegin(GL_LINE_LOOP);
		glVertex3f(-1,-1,-1);
		glVertex3f(1,-1,-1);
		glVertex3f(1,-1,1);
		glVertex3f(-1,-1,1);
	glEnd();
	
	glBegin(GL_LINE_LOOP);
		glVertex3f(-1,1,-1);
		glVertex3f(1,1,-1);
		glVertex3f(1,1,1);
		glVertex3f(-1,1,1);
	glEnd();
	
	glBegin(GL_LINES);
		glVertex3f(-1,-1,-1);
		glVertex3f(-1,1,-1);
		
		glVertex3f(1,-1,-1);
		glVertex3f(1,1,-1);
		
		glVertex3f(1,-1,1);
		glVertex3f(1,1,1);
		
		glVertex3f(-1,-1,1);
		glVertex3f(-1,1,1);		
	glEnd();

}

void TestPSystem::EnableLighting()
{

	GLfloat col[] = {1,1,1,1.0};
	
	GLfloat spec[] = {1,1,1,1.0};
	GLfloat shine[] = {1.0};
	
	GLfloat pos[] = {150,50,10,1};
	
	GLfloat diff[] = {1.0,1.0,1.0,1.0};

	GLfloat lmodel_ambient[] = {0.2,0.2,0.2,1};
	
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, col);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, spec);	
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shine);	
		
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diff);
	glLightfv(GL_LIGHT0, GL_SPECULAR, diff);
	glLightfv(GL_LIGHT0, GL_POSITION, pos);
		
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
	
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_COLOR_MATERIAL);
}

void TestPSystem::Zoom(double amount)
{
	Vector3 dV = (m_Camera - m_LookAt).Normal();
//	dV = dV/amount;
//	m_Camera = m_LookAt + dV;	
	m_Camera -= dV*amount;
	m_LookAt -= dV*amount;
}

void TestPSystem::BeginMainLoop(void)
{
	TestPSystem::m_Instance = this;
	glutMainLoop();
}

void TestPSystem::EnableTimer(int t)
{
	m_MSPerFrame = t;
	glutTimerFunc(t,&TestPSystem::TIMER_CALLBACK,0);
}

int TestPSystem::Time()
{
	return glutGet(GLUT_ELAPSED_TIME);
}

void TestPSystem::PrintString(char* s)
{
	static void* font = {GLUT_BITMAP_9_BY_15};

	if (s && std::strlen(s))
	{
		while (*s)
		{
			glutBitmapCharacter((void*)font,*s);
			s++;
		}
	}
}

void TestPSystem::DisplayFPS()
{
	glColor3f(1,1,1);
	WindowPos2f(m_Width-100,m_Height-15);

	char buffer[10];
	std::sprintf(buffer,"fps: %g",m_FPS);	
	PrintString(buffer);
	
	WindowPos2f(m_Width-109,m_Height-30);
	double mspf = 1000*1/m_FPS; //milliseconds per frame
	std::sprintf(buffer,"mspf: %g",mspf);
	PrintString(buffer);
		
	WindowPos2f(m_Width-109,m_Height-45);
	std::sprintf(buffer,"msps: %g",mspf- m_MSPerFrame);
	PrintString(buffer);
}

void TestPSystem::WindowPos2f( GLfloat x, GLfloat y)
{	
	double z = 0;
	double w = 1;

   GLfloat fx, fy;

   /* Push current matrix mode and viewport attributes */
   glPushAttrib( GL_TRANSFORM_BIT | GL_VIEWPORT_BIT );

   /* Setup projection parameters */
   glMatrixMode( GL_PROJECTION );
   glPushMatrix();
   glLoadIdentity();
   glMatrixMode( GL_MODELVIEW );
   glPushMatrix();
   glLoadIdentity();

   glDepthRange( z, z );
   glViewport( (int) x - 1, (int) y - 1, 2, 2 );

   /* set the raster (window) position */
   fx = x - (int) x;
   fy = y - (int) y;
   glRasterPos4f( fx, fy, 0.0, w );

   /* restore matrices, viewport and matrix mode */
   glPopMatrix();
   glMatrixMode( GL_PROJECTION );
   glPopMatrix();

   glPopAttrib();
}

void TestPSystem::drawFace(ColOrientedFace *face)
{
	glBegin(GL_QUADS);
		Vector3 sidewaysOn2 = face->GetTop().Cross(face->GetNormal()).Normal()*face->GetWidth()/2;
		Vector3 topwaysOn2 = face->GetTop()*face->GetHeight()/2;
	
		Vector3 sideways = sidewaysOn2*2;
		Vector3 topways = topwaysOn2*2;
	
		Vector3 p = face->GetPos()-sidewaysOn2+topwaysOn2;		
		Vector3 p2 = p+sideways;
		Vector3 p3 = p2-topways;
		Vector3 p4 = p3-sideways;

		glTexCoord2f(0,0);
		glNormal3dv(face->GetNormal().GetData());
		glVertex3dv(p.GetData());
		glTexCoord2f(1,0);
		glVertex3dv(p2.GetData());
		glTexCoord2f(1,1);
		glVertex3dv(p3.GetData());
		glTexCoord2f(0,1);
		glVertex3dv(p4.GetData());
	glEnd();
}

void TestPSystem::drawAABB(ColAABB *a, GLuint tex)
{
	std::vector<Vector3*> vf, vb, vt, vbt, vl, vr;
	vf = a->GetFrontFace();
	vb = a->GetBackFace();
	vt = a->GetTopFace();
	vbt = a->GetBottomFace();
	vl = a->GetLeftFace();
	vr = a->GetRightFace();

	glBindTexture(GL_TEXTURE_2D, tex);
	//Front
	glNormal3f(0,0,1);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3dv(vf[0]->GetData());
		glTexCoord2f(1,0);
		glVertex3dv(vf[1]->GetData());
		glTexCoord2f(1,1);
		glVertex3dv(vf[2]->GetData());
		glTexCoord2f(0,1);
		glVertex3dv(vf[3]->GetData());
	glEnd();
	
	//Back
	glNormal3f(0,0,-1);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3dv(vb[0]->GetData());
		glTexCoord2f(1,0);
		glVertex3dv(vb[1]->GetData());
		glTexCoord2f(1,1);
		glVertex3dv(vb[2]->GetData());
		glTexCoord2f(0,1);
		glVertex3dv(vb[3]->GetData());
	glEnd();
	
	//Top
	glNormal3f(0,1,0);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3dv(vt[0]->GetData());
		glTexCoord2f(1,0);
		glVertex3dv(vt[1]->GetData());
		glTexCoord2f(1,1);
		glVertex3dv(vt[2]->GetData());
		glTexCoord2f(0,1);
		glVertex3dv(vt[3]->GetData());
	glEnd();

	//Bottom
	glNormal3f(0,-1,0);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3dv(vbt[0]->GetData());
		glTexCoord2f(1,0);
		glVertex3dv(vbt[1]->GetData());
		glTexCoord2f(1,1);
		glVertex3dv(vbt[2]->GetData());
		glTexCoord2f(0,1);
		glVertex3dv(vbt[3]->GetData());
	glEnd();

	//Left
	glNormal3f(-1,0,0);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3dv(vl[0]->GetData());
		glTexCoord2f(1,0);
		glVertex3dv(vl[1]->GetData());
		glTexCoord2f(1,1);
		glVertex3dv(vl[2]->GetData());
		glTexCoord2f(0,1);
		glVertex3dv(vl[3]->GetData());
	glEnd();

	//Right
	glNormal3f(1,0,0);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3dv(vr[0]->GetData());
		glTexCoord2f(1,0);
		glVertex3dv(vr[1]->GetData());
		glTexCoord2f(1,1);
		glVertex3dv(vr[2]->GetData());
		glTexCoord2f(0,1);
		glVertex3dv(vr[3]->GetData());
	glEnd();
}

/* EMITTER STUFF */

/*
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D,floortex);
	
	glBegin(GL_TRIANGLES);
	glEnd();
	
	glDisable(GL_TEXTURE_2D);
*/	

BasicEmitter::BasicEmitter():CollidableEmitter(2000,1,0.025)
{
	Setup();
}

void BasicEmitter::Draw()
{
	int i;
	
	Vector3 pos;
	for(i=0; i<m_CurrentParticles; i++)
	{
		pos = m_Particles[i]->GetPos();
		
		glTranslatef(pos[0],pos[1],pos[2]);	
		glutSolidSphere(0.025,8,8);		
		glTranslatef(-pos[0],-pos[1],-pos[2]);	
	}		
}


