#include "PSystem.h"
#include "testbed.h"
#include "RigidStrut.h"
#include "MPWorld.h"

#include <iostream>

using namespace Penguin;

class TestPSystem: public TestBed
{
	PSystem m_System;
	
	DampedSpring* m_Spring[3];
	DampedSpring* m_ESpring[3];
	
	public:
	TestPSystem(int argc,char**argv);
	virtual void Render();
	virtual void Update();
		
	void DrawParticles();	
	void DrawSprings();
};

void TestPSystem::DrawSprings()
{
	int i;
	for (i=0; i<3; i++)
	{
	glBegin(GL_LINES);
		glColor3f(1,0,0);
		glVertex3dv(m_Spring[i]->GetOne()->GetPos().GetData());	
		glVertex3dv(m_Spring[i]->GetTwo()->GetPos().GetData());
		
		glColor3f(0,0,1);
		glVertex3dv(m_ESpring[i]->GetOne()->GetPos().GetData());	
		glVertex3dv(m_ESpring[i]->GetTwo()->GetPos().GetData());
	glEnd();
	}
}

void TestPSystem::DrawParticles()
{
	glColor3f(1,0,0);

	std::vector<Particle*>* p = m_System.GetWorld("MPTest")->GetParticles();
	std::vector<Particle*>::iterator it = p->begin();
	
	glBegin(GL_POINTS);
		for(;it!=p->end();++it)
		{
			glVertex3dv((*it)->GetPos().GetData());
		}	
	glEnd();
	
	glColor3f(1,1,0);

	std::vector<ParametricParticle*>* p2 = m_System.GetWorld("MPTest")->GetParametricParticles();
	std::vector<ParametricParticle*>::iterator it2 = p2->begin();
	
	glBegin(GL_POINTS);
		for(;it2!=p2->end();++it2)
		{
			glVertex3dv((*it2)->GetPos().GetData());
		}	
	glEnd();
	
	
	glColor3f(0,0,1);

	p = m_System.GetWorld("ETest")->GetParticles();
	it = p->begin();
	
	glBegin(GL_POINTS);
		for(;it!=p->end();++it)
		{
			glVertex3dv((*it)->GetPos().GetData());
		}	
	glEnd();
	
	glColor3f(0,1,1);

	p2 = m_System.GetWorld("ETest")->GetParametricParticles();
	it2 = p2->begin();
	
	glBegin(GL_POINTS);
		for(;it2!=p2->end();++it2)
		{
			glVertex3dv((*it2)->GetPos().GetData());
		}	
	glEnd();
	
}

int main(int argc, char** argv)
{
	TestPSystem test(argc,argv);
	test.EnableTimer(20); // enable update every 20 milliseconds
	test.BeginMainLoop();
	
	return 0;
}


class StableParticle: public ParametricParticle
{
	public:
	StableParticle(Vector3 pos){SetPos(pos);}
	
	void Simulate(double t){}
};

TestPSystem::TestPSystem(int argc,char**argv):TestBed(argc,argv)
{
	World* mpWorld = m_System.AddWorld("MPTest", new MPWorld(2,0));
	World* eWorld = m_System.AddWorld("ETest", new World(2,0));	
	
	double dX = 0.2;
	
	/* Add things to mp world */
	Particle* particles[4];		
		
	particles[0] = mpWorld->AddParametricParticle(new StableParticle(Vector3(-dX,1,0)));
	particles[1] = mpWorld->AddParticle(new Particle(Vector3(-dX,0,0)));	
	particles[2] = mpWorld->AddParticle(new Particle(Vector3(-dX,-1,0)));
	particles[3] = mpWorld->AddParticle(new Particle(Vector3(dX-0.5,-1,0)));

	m_Spring[0] = (DampedSpring*) mpWorld->AddForce(new DampedSpring(particles[0],particles[1],1,10,0));
	m_Spring[1] = (DampedSpring*) mpWorld->AddForce(new DampedSpring(particles[1],particles[2],0.5,10,0));
	m_Spring[2] = (DampedSpring*) mpWorld->AddForce(new DampedSpring(particles[2],particles[3],1,10,0));
	
	/* Add things to e world */
	
	particles[0] = eWorld->AddParametricParticle(new StableParticle(Vector3(dX,1,0)));
	particles[1] = eWorld->AddParticle(new Particle(Vector3(dX,0,0)));	
	particles[2] = eWorld->AddParticle(new Particle(Vector3(dX,-1,0)));
	particles[3] = eWorld->AddParticle(new Particle(Vector3(dX+0.5,-1,0)));

	m_ESpring[0] = (DampedSpring*) eWorld->AddForce(new DampedSpring(particles[0],particles[1],1,10,0));
	m_ESpring[1] = (DampedSpring*) eWorld->AddForce(new DampedSpring(particles[1],particles[2],0.5,10,0));
	m_ESpring[2] = (DampedSpring*) eWorld->AddForce(new DampedSpring(particles[2],particles[3],1,10,0));
}

void TestPSystem::Render(void)
{
	glPointSize(5);

	DrawSprings();	
	DrawParticles();	
}

void TestPSystem::Update(void)
{
	static double dt = m_MSPerFrame/1000.0;
	m_System.Simulate(dt);
	
	m_CenterY += dt*.1;
}

