/*
*
* A "Newton's Cradle" demo.
* Ben Porter. 9/9/04
*
*/

#include "PSystem.h"
#include "testbed.h"
#include "Collision.h"

#include <cmath>

using namespace Penguin;

class Cradle: public Object
{
	public:
	
	Cradle();
	virtual ~Cradle(){}
	
	void Draw();
};

class MyApp: public TestBed
{
	PSystem m_System;	
	Cradle* m_Cradle;

	public:
	MyApp(int argc, char** argv);
	
	virtual void Render();
	virtual void Update();
};

int main(int argc, char** argv)
{
	MyApp app(argc,argv);
	app.EnableTimer(10);
	app.BeginMainLoop();
	return 0;	
}

MyApp::MyApp(int argc, char** argv):TestBed(argc,argv)
{
	Setup();
	World* world = m_System.AddWorld("Test", new World(5,0.01));
	world->ManageObject(m_Cradle = new Cradle(),"Cradle");		
}

void MyApp::Render(void)
{
	glPointSize(5);
	SetViewport(-5,5,-5,5);	
	m_Cradle->Draw();
}

void MyApp::Update(void)
{
	static double dt = m_MSPerFrame/1000.0;
	m_System.Simulate(dt);
}

Cradle::Cradle()
{
	double radius = 0.5;
	double diameter = 2*radius;	
	
	double x = -radius-diameter;
	double y = -0.5;
	
	double length = 2;
	double bounce = 1.01;
	
	double k = 100;
	double c = 10;
	
	int i;
	for (i=0; i<4; i++)
	{/*
		CollidableParticle* ball = new CollidableParticle(radius);
		ball->SetPos(Vector3(x,y,0));
		*/
		Particle* ptcl = new Particle(Vector3(x,y,0));	
		
		ColSphere* ball = new ColSphere(ptcl,radius);		
		ball->SetCoefficientOfRestitution(bounce);
	
		AddParticle(ptcl);
		AddCollisionStruct(ball);
		
		StableParticle* pivot = new StableParticle(Vector3(x,y+length,0));
		AddParametricParticle(pivot);
		
		DampedSpring* wire = new DampedSpring(pivot,ptcl,length,k,c); //one,two,length,k,c
		AddForce(wire);
		
		x += diameter;
	}
	
	double angleRaised = 1.5; //radians (1 rad = 57 deg)
	
	Particle* ptcl = new Particle(Vector3(x+length*sin(angleRaised),y+length-length*cos(angleRaised),0));
	
	ColSphere* raisedBall = new ColSphere(ptcl,radius);
	raisedBall->SetCoefficientOfRestitution(bounce);
	
	StableParticle* pivot = new StableParticle(Vector3(x,y+length,0));
	DampedSpring* wire = new DampedSpring(pivot,ptcl,length,k,c); //one,two,length,k,c
	
	AddParticle(ptcl);
	AddCollisionStruct(raisedBall);		
	AddParametricParticle(pivot);	
	AddForce(wire);
}

void Cradle::Draw()
{
	static GLfloat colours[5][4] = {{1,1,1,1},{1,0,0,1},{0,1,0,1},{0,0,1,1},{0.2,1,0.6,1}};

	FIter wire_it = m_Forces.begin();
	FIter wire_end = m_Forces.end();

	glColor3f(1,1,1);
	
	for (;wire_it!=wire_end;++wire_it)
	{
		DampedSpring* ds = static_cast<DampedSpring*>(*wire_it);

		Vector3& pos1 = ds->GetOne()->GetPos();
		Vector3& pos2 = ds->GetTwo()->GetPos();

		glBegin(GL_LINES);		
			glVertex3dv(pos1.GetData());
			glVertex3dv(pos2.GetData());
		glEnd();
	}

	ColIter it = GetCollisionStructsBegin();
	ColIter end = GetCollisionStructsEnd();
	
	int i=0;
	
	for(;it!=end;++it)
	{
		Vector3& pos = (static_cast<ColSphere*>(*it))->GetTarget()->GetPos();
		
		glColor4fv(colours[i++]);
		
		glTranslatef(pos[0],pos[1],pos[2]);
	
		glutSolidSphere((static_cast<ColSphere*>(*it))->GetRadius(),32,32);
		
		glTranslatef(-pos[0],-pos[1],-pos[2]);	
	}	
}


