#include "pool.h"
#include "IL/ilut.h"

#include <iostream>

using namespace Penguin;

double speed_factor = 0.5; //i.e. how much should we update the world relative to the screen

TestPSystem* TestPSystem::m_Instance = NULL;

GLuint texname;
GLuint shadowtex;

PoolTable* g_table;

//NUM_BALLS doesn't include the main ball
#define NUM_BALLS 9

Ball* g_ball; //main ball
Ball* g_balls[NUM_BALLS]; //named "0", "1", "2", "3", etc....
bool g_valid_balls[NUM_BALLS];


int main(int argc, char** argv)
{
	glutInit(&argc,argv);
	
	ilInit();
	iluInit();
	ilutRenderer(ILUT_OPENGL);	

	TestPSystem test;
	test.EnableTimer(5);
	test.BeginMainLoop();	
	return 0;
}

TestPSystem::TestPSystem():m_Width(500),m_Height(500), m_NumFrames(0), m_FPS(0), m_MSSinceLastDraw(0), m_MSPerFrame(0), m_IsUpdating(true),m_TrackingBall(0)
{
	InitialiseGlut();
	InitialiseGL();

	world = m_System.AddWorld("Test", new World(5,0.0));
	if (!world)
	{
		std::cout << "Can't create world!" << std::endl;
	}
	else
	{
		g_table = new PoolTable();
		g_ball = new Ball(Vector3(0,0.3,1),0.25,Vector3(1,1,1));
		SetTrackingBall(g_ball);

		/* Arrange the balls in a channel 9 logo style.... */
		
		double r = 0.25; //radius
		double w /*width of formation*/ = 2;
		double zc = -.5; //zcenter
		double y = 0.3; //start height

		/* 0 1 2 
		   3 4 5
			 6 7 8 */
	
		Vector3 colours[NUM_BALLS] = {	Vector3(0,0,1), //blue
																		Vector3(0,1,0), //green
																		Vector3(1,0,0), //red
																		Vector3(1,1,0), //yellow
																		Vector3(0,0,0), //black
																		Vector3(0,0.8,1), //aqua
																		Vector3(0.2,0.2,0.2), //grey
																		Vector3(1,0,1), //magenta
																		Vector3(0.4,0.2,0.8)}; //who knows?	
		double top = zc-w;
		double left = -w/2;
		double ds = w/2; //distance between ball centers
		
		int i,j;
		for(i=0; i<3; i++) //row
		{
			for(j=0; j<3; j++) //col
			{
				int index = i*3 + j;
				g_balls[index] = new Ball(Vector3(left+ds*j,y,top+ds*i),r,colours[index]);
				char name[2];
				name[1] = 0;
				name[0] = '0'+index;
				world->ManageObject(g_balls[index],name);
			}
		}

		world->ManageObject(g_table,"Table");
		world->ManageObject(g_ball,"Ball");

		SetCamera(Vector3(1.75,2.3,-3.5));
		SetLookAt(Vector3(0,0,0));
		EnableLighting();
	}	
}
	
void TestPSystem::Render(void)
{
	g_table->Draw();
	g_ball->Draw();

	int i;
	for(i=0; i<NUM_BALLS;++i)
	{
		g_balls[i]->Draw();
	}
}

void TestPSystem::Update(void)
{
	static double dt = m_MSPerFrame/1000.0;
	m_System.Simulate(dt*speed_factor);
}

void TestPSystem::KeyListener(unsigned char key, int x, int y)
{
	if (key==' ') m_IsUpdating = !m_IsUpdating;	
	if (key=='a') RotateAroundY(0.1);	
	else if (key=='d') RotateAroundY(-0.1);	
	else if (key=='w') Zoom(1.1);
	else if (key=='s') Zoom(0.9);
	else if (key=='r') AddVecToCamera(Vector3(0,0.1,0));
	else if (key=='f') AddVecToCamera(Vector3(0,-0.1,0));
	else if (key=='z') 
	{
		if (m_TrackingBall) //If we are tracking a ball then give it a whack!
		{
			//The direction will be in the direction we are looking... i.e.  cam_pos - ball_pos
			double power = 50; //Maybe implement a variable power system?
			Vector3 whack = (m_TrackingBall->GetPos() - m_Camera).Normal() * power;
			whack[1] = 0;
			world->AddImpulse(m_TrackingBall->GetParticle(),whack);
		}
	}
}

/* SET UP GLUT AND OPENGL */
void TestPSystem::InitialiseGlut()
{
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

	//Create a single window
	glutInitWindowPosition(100,100);
	glutInitWindowSize(m_Width,m_Height);		
	glutCreateWindow("Simple physics engine.");		
	
	//Callbacks
	glutDisplayFunc(&TestPSystem::DISPLAY_CALLBACK);
	glutReshapeFunc(&TestPSystem::RESHAPE_CALLBACK);
	glutKeyboardFunc(&TestPSystem::KEYBOARD_CALLBACK);	
	glutMouseFunc(&TestPSystem::MOUSECLICK_CALLBACK);
	glutMotionFunc(&TestPSystem::MOUSEMOVE_CALLBACK);
	glutPassiveMotionFunc(&TestPSystem::PASSIVEMOUSEMOVE_CALLBACK);
	
	glutSetCursor(GLUT_CURSOR_CROSSHAIR);
}

void TestPSystem::InitialiseGL()
{
	glClearColor(1,1,1,1);	
	double aspect = (double)m_Height/m_Width;	
	Set3dViewport(aspect);
	
	texname = ilutGLLoadImage("cloth.png");
	shadowtex = ilutGLLoadImage("shadow.png");
}

void TestPSystem::Reshape(int w, int h)
{
	m_Width = w;
	m_Height = h;
	
	double aspect = (double)m_Height/m_Width;
		
	glViewport(0,0,w,h);
	
	Set3dViewport(aspect);
}

void TestPSystem::Display(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	
	DisplayFPS();	
	
	glEnable(GL_DEPTH_TEST);	
	glEnable(GL_LIGHTING);

	Vector3 lookat;

	if (m_TrackingBall)
		lookat = m_TrackingBall->GetPos();
	else 
		lookat = m_LookAt;

	gluLookAt(m_Camera[0],m_Camera[1],m_Camera[2],lookat[0],lookat[1],lookat[2],0,1,0);
	
	Render();
	
	glutSwapBuffers();
}

void TestPSystem::Set3dViewport(double aspect)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60,1/aspect,.1,1000); 
	
	glMatrixMode(GL_MODELVIEW);
}

void TestPSystem::RotateAroundY(double angle)
{
	Vector3 dV;

	if (!m_TrackingBall)
	{
		dV = m_Camera - m_LookAt;
	}
	else
	{
		dV = m_Camera - m_TrackingBall->GetPos();
	}
	
	double x = dV[0];
	double z = dV[2];
	
	x = dV[0]*cos(angle) + dV[2]*sin(angle);
	z = -dV[0]*sin(angle) + dV[2]*cos(angle);
	
	dV[0] = x;
	dV[2] = z;
	
	if (!m_TrackingBall) m_Camera = dV + m_LookAt;	
	else m_Camera = dV + m_TrackingBall->GetPos();
}

void TestPSystem::RenderGuides()
{
	glColor3f(0.5,0.5,0.5);
	
	glBegin(GL_LINE_LOOP);
		glVertex3f(-1,-1,-1);
		glVertex3f(1,-1,-1);
		glVertex3f(1,-1,1);
		glVertex3f(-1,-1,1);
	glEnd();
	
	glBegin(GL_LINE_LOOP);
		glVertex3f(-1,1,-1);
		glVertex3f(1,1,-1);
		glVertex3f(1,1,1);
		glVertex3f(-1,1,1);
	glEnd();
	
	glBegin(GL_LINES);
		glVertex3f(-1,-1,-1);
		glVertex3f(-1,1,-1);
		
		glVertex3f(1,-1,-1);
		glVertex3f(1,1,-1);
		
		glVertex3f(1,-1,1);
		glVertex3f(1,1,1);
		
		glVertex3f(-1,-1,1);
		glVertex3f(-1,1,1);		
	glEnd();

}

void TestPSystem::EnableLighting()
{
	GLfloat col[] = {1,1,1,1.0};
	GLfloat spec[] = {1,1,1,1.0};
	GLfloat shine[] = {20.0};
	
	GLfloat pos[] = {-15,10,3,1};
	GLfloat pos2[] = {0,0.4,5,1};
	GLfloat pos3[] = {15,-10,3,1};
	
	GLfloat diff[] = {1.0,1.0,1.0,1.0};
	GLfloat lmodel_ambient[] = {0.2,0.2,0.2,1};
	
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, col);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, spec);	
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shine);	
		
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diff);
	glLightfv(GL_LIGHT0, GL_POSITION, pos);
	
	glLightfv(GL_LIGHT1, GL_DIFFUSE, diff);
	glLightfv(GL_LIGHT1, GL_POSITION, pos2);
	
	glLightfv(GL_LIGHT2, GL_DIFFUSE, diff);
	glLightfv(GL_LIGHT2, GL_POSITION, pos3);
	
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
	
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);
	glEnable(GL_LIGHT2);
	
	glEnable(GL_COLOR_MATERIAL);
}

void TestPSystem::Zoom(double amount)
{
	Vector3 dV = m_Camera - m_LookAt;
	dV = dV/amount;
	m_Camera = m_LookAt + dV;	
}

void TestPSystem::BeginMainLoop(void)
{
	TestPSystem::m_Instance = this;
	glutMainLoop();
}

void TestPSystem::EnableTimer(int t)
{
	m_MSPerFrame = t;
	glutTimerFunc(t,&TestPSystem::TIMER_CALLBACK,0);
}

int TestPSystem::Time()
{
	return glutGet(GLUT_ELAPSED_TIME);
}

void TestPSystem::PrintString(char* s)
{
	static void* font = {GLUT_BITMAP_9_BY_15};

	if (s && std::strlen(s))
	{
		while (*s)
		{
			glutBitmapCharacter((void*)font,*s);
			s++;
		}
	}
}

void TestPSystem::DisplayFPS()
{
	glColor3f(1,1,1);
	WindowPos2f(m_Width-100,m_Height-15);

	char buffer[10];
	std::sprintf(buffer,"fps: %g",m_FPS);	
	PrintString(buffer);
	
	WindowPos2f(m_Width-109,m_Height-30);
	double mspf = 1000*1/m_FPS; //milliseconds per frame
	std::sprintf(buffer,"mspf: %g",mspf);
	PrintString(buffer);
		
	WindowPos2f(m_Width-109,m_Height-45);
	std::sprintf(buffer,"msps: %g",mspf- m_MSPerFrame);
	PrintString(buffer);
}

void TestPSystem::WindowPos2f( GLfloat x, GLfloat y)
{	
	double z = 0;
	double w = 1;

   GLfloat fx, fy;

   /* Push current matrix mode and viewport attributes */
   glPushAttrib( GL_TRANSFORM_BIT | GL_VIEWPORT_BIT );

   /* Setup projection parameters */
   glMatrixMode( GL_PROJECTION );
   glPushMatrix();
   glLoadIdentity();
   glMatrixMode( GL_MODELVIEW );
   glPushMatrix();
   glLoadIdentity();

   glDepthRange( z, z );
   glViewport( (int) x - 1, (int) y - 1, 2, 2 );

   /* set the raster (window) position */
   fx = x - (int) x;
   fy = y - (int) y;
   glRasterPos4f( fx, fy, 0.0, w );

   /* restore matrices, viewport and matrix mode */
   glPopMatrix();
   glMatrixMode( GL_PROJECTION );
   glPopMatrix();

   glPopAttrib();
}

/***************************** POOL TABLE STUFF ********************************/
/*******************************************************************************/



Ball::Ball(Vector3 pos, REAL radius, Vector3 colour):_radius(radius),_colour(colour)
{
	_particle = AddParticle(new Particle(pos));	
	ColSphere* s = new ColSphere(_particle,radius);
	s->SetCOF(0.01);
	AddCollisionStruct(s);
}

void Ball::Draw()
{
	Vector3& pos = _particle->GetPos(); 
	
	static double eps = RandomREAL(0.001,0.01);
	double fudge = 0.2;

	glTranslatef(pos[0],pos[1]+0.01,pos[2]);
		glColor3f(_colour[0],_colour[1],_colour[2]);
		glutSolidSphere(_radius,32,32);	

	glTranslatef(0,-pos[1]-0.01,0);

	glColor3f(1,1,1);

	glDisable(GL_LIGHTING);
	glEnable(GL_BLEND);
	glBlendEquation(GL_MIN);

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D,shadowtex);

	double r = _radius+fudge;

	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3f(-r,eps,-r);
	
		glTexCoord2f(1,0);
		glVertex3f(r,eps,-r);
	
		glTexCoord2f(1,1);
		glVertex3f(r,eps,r);
		
		glTexCoord2f(0,1);
		glVertex3f(-r,eps,r);		
	glEnd();
	
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);	
	glEnable(GL_LIGHTING);
	
	glTranslatef(-pos[0],0,-pos[2]);
}

PoolTable::PoolTable()
{
	y_plane = 0;
	hole_radius = 0.5;
	table_width = 4;
	table_length = 6;

	REAL w2 = table_width/2;
	REAL l2 = table_length/2;

	// Main table plane
	ColPlane* p = new ColPlane(Vector3(0,1,0),Vector3(0,y_plane,0));
	p->SetCOF(0.03);
	AddCollisionStruct(p);

	/*
     <---W--->  
		     4
  ^  O-------O
  |  |       |
  H 1O       O2
  |  |       |
  V  O-------O
	       3
				 */

	/* Sides 1 and 2 */
	AddCollisionStruct(p = new ColPlane(Vector3(1,0,0),Vector3(-w2,0,0))); // 1
	p->SetCOF(0.03);
	AddCollisionStruct(p = new ColPlane(Vector3(-1,0,0),Vector3(w2,0,0))); // 2
	p->SetCOF(0.03);

	/* Sides 3 and 4 */
	AddCollisionStruct(p = new ColPlane(Vector3(0,0,-1),Vector3(0,0,l2))); // 3
	p->SetCOF(0.03);
	AddCollisionStruct(p = new ColPlane(Vector3(0,0,1),Vector3(0,0,-l2))); // 4
	p->SetCOF(0.03);	
}

void PoolTable::Draw()
{
	//Draw table plane
	REAL w2 = table_width/2;
	REAL l2 = table_length/2;

	glNormal3f(0,1,0);
	glColor3f(0,0.9,0);
	glBegin(GL_QUADS);
		glVertex3f(-w2,y_plane,-l2);
		glVertex3f(w2,y_plane,-l2);
		glVertex3f(w2,y_plane,l2);
		glVertex3f(-w2,y_plane,l2);
	glEnd();

	//Draw table sides
	REAL bw /*border_width*/ = 0.5;
	REAL bh /*border_height*/ = 0.35;
	REAL top = y_plane + bh;
	
	glBegin(GL_QUADS);
		//LEFT SIDE (TOP FACE)
		glNormal3f(0,1,0);
		glVertex3f(-w2-bw,top,-l2-bw);
		glVertex3f(-w2,top,-l2);
		glVertex3f(-w2,top,l2);
		glVertex3f(-w2-bw,top,l2+bw);

		//LEFT SIDE (INNER FACE)
		glNormal3f(1,0,0);
		glVertex3f(-w2,top,-l2);
		glVertex3f(-w2,top-bh,-l2);
		glVertex3f(-w2,top-bh,l2);
		glVertex3f(-w2,top,l2);
		
		//LEFT SIDE (OUTER FACE)
		glNormal3f(-1,0,0);
		glVertex3f(-w2-bw,top,-l2-bw);
		glVertex3f(-w2-bw,top,l2+bw);
		glVertex3f(-w2-bw,top-bh,l2+bw);
		glVertex3f(-w2-bw,top-bh,-l2-bw);
	
		//RIGHT SIDE (TOP FACE)
		glNormal3f(0,1,0);
		glVertex3f(w2+bw,top,-l2-bw);
		glVertex3f(w2+bw,top,l2+bw);
		glVertex3f(w2,top,l2);
		glVertex3f(w2,top,-l2);

		//RIGHT SIDE (INNER FACE)
		glNormal3f(-1,0,0);
		glVertex3f(w2,top-bh,-l2);
		glVertex3f(w2,top,-l2);
		glVertex3f(w2,top,l2);
		glVertex3f(w2,top-bh,l2);
		
		//RIGHT SIDE (OUTER FACE)
		glNormal3f(1,0,0);
		glVertex3f(w2+bw,top,-l2-bw);
		glVertex3f(w2+bw,top-bh,-l2-bw);
		glVertex3f(w2+bw,top-bh,l2+bw);
		glVertex3f(w2+bw,top,l2+bw);

		//TOP SIDE (TOP FACE)
		glNormal3f(0,1,0);
		glVertex3f(-w2-bw,top,-l2-bw);
		glVertex3f(w2+bw,top,-l2-bw);
		glVertex3f(w2,top,-l2);
		glVertex3f(-w2,top,-l2);
		
		//TOP SIDE (INNER FACE)
		glNormal3f(0,0,1);
		glVertex3f(-w2,top,-l2);
		glVertex3f(w2,top,-l2);
		glVertex3f(w2,top-bh,-l2);
		glVertex3f(-w2,top-bh,-l2);

		//TOP SIDE (OUTER FACE)
		glNormal3f(0,0,-1);
		glVertex3f(-w2-bw,top,-l2-bw);
		glVertex3f(-w2-bw,top-bh,-l2-bw);
		glVertex3f(w2+bw,top-bh,-l2-bw);
		glVertex3f(w2+bw,top,-l2-bw);
		
		//BOTTOM SIDE (TOP FACE)
		glNormal3f(0,1,0);
		glVertex3f(-w2-bw,top,l2+bw);
		glVertex3f(-w2,top,l2);
		glVertex3f(w2,top,l2);
		glVertex3f(w2+bw,top,l2+bw);
		
		//BOTTOM SIDE (INNER FACE)
		glNormal3f(0,0,-1);
		glVertex3f(-w2,top,l2);
		glVertex3f(-w2,top-bh,l2);
		glVertex3f(w2,top-bh,l2);
		glVertex3f(w2,top,l2);

		//BOTTOM SIDE (OUTER FACE)
		glNormal3f(0,0,1);
		glVertex3f(-w2-bw,top,l2+bw);
		glVertex3f(w2+bw,top,l2+bw);
		glVertex3f(w2+bw,top-bh,l2+bw);
		glVertex3f(-w2-bw,top-bh,l2+bw);

	glEnd();
	
}
