/*
 * Tests: PSystem, World, Particle and Force
 * 
 */
 
#include "PSystem.h"
#include "testbed.h"
#include "RigidStrut.h"
#include "MPWorld.h"
#include "Collision.h"
#include <ctime>

#include <iostream>

using namespace Penguin;

class TestPSystem: public TestBed
{
	PSystem m_System;
	
	ColSphere* m_Spheres[2];
		
	public:
	TestPSystem(int argc,char**argv);
	virtual void Render();
	virtual void Update();
		
	void DrawParticles();	
};

void TestPSystem::DrawParticles()
{
	int i;
	for (i=0; i<2; i++)
	{	
		glColor3f(0.5,1,1);

		Vector3& pos = m_Spheres[i]->GetTarget()->GetPos();

		glPushMatrix();
		glTranslatef(pos[0],pos[1],pos[2]);
		glutSolidSphere(m_Spheres[i]->GetRadius(),16,16);	
		glPopMatrix();		
	}
}

int main(int argc, char** argv)
{
	srand(time(NULL));
	TestPSystem test(argc,argv);
	test.EnableTimer(10); // enable update every 10 milliseconds
	test.BeginMainLoop();
	
	return 0;
}
double randomDBL()
{
        return double(std::rand())/RAND_MAX;
}

TestPSystem::TestPSystem(int argc,char**argv):TestBed(argc,argv)
{
	Setup();
	
	World* eWorld = m_System.AddWorld("Test", new World(5));
	Object* tempObj = new Object();
	eWorld->ManageObject(tempObj,"t1");
	
	Particle* p = new Particle(Vector3(0.1,1,0));	
	m_Spheres[0] = new ColSphere(p,0.2);
	tempObj->AddParticle(p);
	tempObj->AddCollisionStruct(m_Spheres[0]);
	m_Spheres[0]->SetCoefficientOfRestitution(0.3);
	
	StableParticle* p2 = new StableParticle(Vector3(0,-1.3,0));	
	m_Spheres[1] = new ColSphere(p2,0.5);
	m_Spheres[1]->SetMoveable(false);
	tempObj->AddParametricParticle(p2);
	tempObj->AddCollisionStruct(m_Spheres[1]);
	
	m_IsUpdating = false; //start the simulation paused
}

void TestPSystem::Render(void)
{		
	SetViewport(-2,2,-2,2);
	DrawParticles();	
}

void TestPSystem::Update(void)
{
	static double dt = m_MSPerFrame/1000.0;	
	m_System.Simulate(dt);	
}


