/*
 * Tests: PSystem, World, Particle and Force
 * 
 */
 
#include "PSystem.h"
#include "testbed.h"
#include "RigidStrut.h"
#include "MPWorld.h"
#include "Collision.h"
#include <ctime>

#include <iostream>

using namespace Penguin;

const int NUMPARTICLES = 40;

class TestPSystem: public TestBed
{
	PSystem m_System;
	
	Particle* m_EulerParticles[3];
	Particle* m_MidPtParticles[3];
	DampedSpring* m_Springs[4];
	
	public:
	TestPSystem(int argc,char**argv);
	virtual void Render();
	virtual void Update();
		
	void DrawParticles();	
};

void TestPSystem::DrawParticles()
{
	int i;
	for (i=0; i<3; i++)
	{	
		glColor3f(0.5,1,1);

		Vector3& pos = m_EulerParticles[i]->GetPos();

		glPushMatrix();
		glTranslatef(pos[0],pos[1],pos[2]);
		glutSolidSphere(0.1,16,16);	
		glPopMatrix();		
	}
	
	for (i=0; i<3; i++)
	{	
		glColor3f(1,0.5,1);

		Vector3& pos = m_MidPtParticles[i]->GetPos();

		glPushMatrix();
		glTranslatef(pos[0],pos[1],pos[2]);
		glutSolidSphere(0.1,16,16);	
		glPopMatrix();		
	}
	
	glBegin(GL_LINES);
	for (i=0; i<4; i++)
	{
		Vector3 one = m_Springs[i]->GetOne()->GetPos();
		Vector3 two = m_Springs[i]->GetTwo()->GetPos();
		
		glVertex3dv(one.GetData());
		glVertex3dv(two.GetData());
	}
	glEnd();
}

int main(int argc, char** argv)
{
	srand(time(NULL));
	TestPSystem test(argc,argv);
	test.EnableTimer(10); // enable update every 20 milliseconds
	test.BeginMainLoop();
	
	return 0;
}
double randomDBL()
{
        return double(std::rand())/RAND_MAX;
}

TestPSystem::TestPSystem(int argc,char**argv):TestBed(argc,argv)
{
	Setup();
	
	int i;
	for (i=0; i<3; i++)
	{
		if (i==0)
		{
			m_EulerParticles[i] = new StableParticle(Vector3(-0.5,(1-i)*0.5,0));
		}
		else
		{
			m_EulerParticles[i] = new Particle(Vector3(-randomDBL(),(1-i)*0.5,0));
		}
				
		m_EulerParticles[i]->SetMass(1);
	}
	
	for (i=0; i<2; i++)
	{
		m_Springs[i] = new DampedSpring(m_EulerParticles[i],m_EulerParticles[i+1],0.9,100,10);
	}
	
	for (i=0; i<3; i++)
	{
		if (i==0)
		{
			m_MidPtParticles[i] = new StableParticle(Vector3(0.5,(1-i)*0.5,0));
		}
		else
		{
			m_MidPtParticles[i] = new Particle(Vector3(randomDBL(),(1-i)*0.5,0));
		}
				
		m_MidPtParticles[i]->SetMass(1);
	}
	
	for (i=2; i<4; i++)
	{
		m_Springs[i] = new DampedSpring(m_MidPtParticles[i-2],m_MidPtParticles[i-1],0.9,100,10);
	}
	
	World* eWorld = m_System.AddWorld("Test", new World(5));
	Object* tempObj = new Object();
	eWorld->ManageObject(tempObj,"t1");

	tempObj->AddParametricParticle(static_cast<StableParticle*>(m_EulerParticles[0]));
	tempObj->AddParticle(m_EulerParticles[1]);
	tempObj->AddParticle(m_EulerParticles[2]);
	tempObj->AddForce(m_Springs[0]);
	tempObj->AddForce(m_Springs[1]);

  World* mWorld = m_System.AddWorld("Test2", new MPWorld(5));
	Object* tempObj2 = new Object();
	mWorld->ManageObject(tempObj2,"t2");
	tempObj2->AddParametricParticle(static_cast<StableParticle*>(m_MidPtParticles[0]));
	tempObj2->AddParticle(m_MidPtParticles[1]);
	tempObj2->AddParticle(m_MidPtParticles[2]);
	tempObj2->AddForce(m_Springs[2]);
	tempObj2->AddForce(m_Springs[3]);

}

void TestPSystem::Render(void)
{		
	SetViewport(-2,2,-2,2);
	DrawParticles();	
}

void TestPSystem::Update(void)
{
	static double dt = m_MSPerFrame/1000.0;	
	m_System.Simulate(dt);	
}


