/*
 * Tests: PSystem, World, Particle and Force
 * 
 */
 
#include "PSystem.h"
#include "testbed.h"
#include "RigidStrut.h"
#include "MPWorld.h"

#include <iostream>

using namespace Penguin;

class TestPSystem: public TestBed
{
	PSystem m_System;
	
	DampedSpring** m_Spring;
	int numSprings;
	
	public:
	TestPSystem(int argc,char**argv);
	~TestPSystem(){delete[]m_Spring;}
	
	virtual void Render();
	virtual void Update();
		
	void DrawParticles();	
	void DrawSprings();
};

void TestPSystem::DrawSprings()
{
	glColor3f(1,1,0);
	glBegin(GL_LINES);	
		int i;
		for(i=0; i<numSprings; i++)
		{	
			glVertex3dv(m_Spring[i]->GetOne()->GetPos().GetData());	
			glVertex3dv(m_Spring[i]->GetTwo()->GetPos().GetData());
		}
	glEnd();
}

void TestPSystem::DrawParticles()
{
	glColor3f(1,1,1);

	std::vector<Particle*>* p = m_System.GetWorld("Test")->GetParticles();
	std::vector<Particle*>::iterator it = p->begin();
	
	glBegin(GL_POINTS);
		for(;it!=p->end();++it)
		{
			glVertex3dv((*it)->GetPos().GetData());
		}	
	glEnd();
	
	glColor3f(0,1,1);

	std::vector<ParametricParticle*>* p2 = m_System.GetWorld("Test")->GetParametricParticles();
	std::vector<ParametricParticle*>::iterator it2 = p2->begin();
	
	glBegin(GL_POINTS);
		for(;it2!=p2->end();++it2)
		{
			glVertex3dv((*it2)->GetPos().GetData());
		}	
	glEnd();
}

int main(int argc, char** argv)
{
	TestPSystem test(argc,argv);
	test.EnableTimer(10);
	test.BeginMainLoop();
	
	return 0;
}

TestPSystem::TestPSystem(int argc,char**argv):TestBed(argc,argv)
{
	Setup();

	World* world = m_System.AddWorld("Test", new MPWorld(5,0.01));
	if (!world)
	{
		std::cout << "Can't create world!" << std::endl;
	}
	else
	{	
		/* Create a Cloth Patch */
						
		Particle* particles[10][10];		
		
		int i,j;
		for (i=0; i<10; i++)
		{
			for (j=0; j<10; j++)
			{
				if (j==9 && (i==0||i==9))
				{
					if (i==0)
					{
				
						particles[i][j] = world->AddParametricParticle(new StableParticle(Vector3(i/5.-1,j/5.-1,0)));
					}
					else
					{
						particles[i][j] = world->AddParametricParticle(new CircleParticle(Vector3(i/5.-1,j/5.-1,0),0.5,1));
						
					}
				}
				else
				{
					particles[i][j] = world->ManageParticle(new Particle(Vector3(i/5.-1,j/5.-1,0)));
				}
			}
		}
				
		numSprings = 180+162; /* 90 horiz, 90 vert, 81*2 diagonal */

		double horiz_k = 1000;
		double horiz_c = 10;		
		
		double vert_k = horiz_k;
		double vert_c = horiz_c;
		
		double diag_k = 100;
		double diag_c = 10;
		
		m_Spring = new DampedSpring*[numSprings];
		
		/* horizontal springs */
		
		for(i=0; i<9; i++)
		{
			for(j=0; j<10; j++)
			{
				m_Spring[i*10+j] = (DampedSpring*) world->AddForce(new DampedSpring(particles[i][j],particles[i+1][j],.2,horiz_k,horiz_c));
			}
		}
		
		/* vertical springs */
		
		for(i=0; i<10; i++)
		{
			for(j=0; j<9; j++)
			{
				m_Spring[90 + i*9+j] = (DampedSpring*) world->AddForce(new DampedSpring(particles[i][j],particles[i][j+1],.2,vert_k,vert_c));
			}
		}	
		
		/* diagonal springs */
		
		double diag = sqrt(0.2*0.2);
		
		for(i=0; i<9; i++)
		{
			for(j=0; j<9; j++)
			{
				m_Spring[180 + i*9+j] = (DampedSpring*) world->AddForce(new DampedSpring(particles[i][j],particles[i+1][j+1],diag,diag_k,diag_c));
				m_Spring[180 + 81 + (i*9+j)] = (DampedSpring*) world->AddForce(new DampedSpring(particles[i+1][j],particles[i][j+1],diag,diag_k,diag_c));				
			}
		}
		
	}	
}

void TestPSystem::Render(void)
{
	glPointSize(5);
	SetViewport(-2,2,-2,2);
	
	DrawSprings();	
	DrawParticles();
}

void TestPSystem::Update(void)
{
	static double dt = m_MSPerFrame/1000.0;
	m_System.Simulate(dt);
}


