
#include "testbed.h"
#include <cstring>
#include <cstdio>

TestBed* TestBed::m_Instance = 0;

TestBed::TestBed(int argc, char** argv)
:m_Width(500),m_Height(500),m_CenterX(0), m_CenterY(0), m_NumFrames(0), m_FPS(0), m_MSSinceLastDraw(0), m_MSPerFrame(0), m_IsUpdating(true)
{
	glutInit(&argc,argv);
}

TestBed::TestBed(int argc, char** argv,int w, int h)
:m_Width(w),m_Height(w), m_CenterX(0), m_CenterY(0), m_NumFrames(0), m_FPS(0), m_MSSinceLastDraw(0), m_MSPerFrame(0), m_IsUpdating(true)
{
	glutInit(&argc,argv);
}

TestBed::~TestBed(){};

void TestBed::Setup()
{
	GlutSetup();
	GLSetup();
}

void TestBed::GlutSetup(void)
{
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

	glutInitWindowPosition(100,100);
	glutInitWindowSize(m_Width,m_Height);
		
	glutCreateWindow("pEngine");		
	
	glutDisplayFunc(&TestBed::DISPLAY_CALLBACK);
	glutReshapeFunc(&TestBed::RESHAPE_CALLBACK);
	glutKeyboardFunc(&TestBed::KEYBOARD_CALLBACK);
	
	glutMouseFunc(&TestBed::MOUSECLICK_CALLBACK);
	glutMotionFunc(&TestBed::MOUSEMOVE_CALLBACK);
	glutPassiveMotionFunc(&TestBed::PASSIVEMOUSEMOVE_CALLBACK);
	
	glutSetCursor(GLUT_CURSOR_CROSSHAIR);
	
}

void TestBed::GLSetup()
{
	glClearColor(0,0,0,0);
	
	double aspect = (double)m_Height/m_Width;
	
	m_Left = -1;
	m_Right = 1;
	m_Bottom = -aspect;
	m_Top = aspect;
	
	SetViewport(-1,1,-aspect,aspect);
}

void TestBed::Reshape(int w, int h)
{
	m_Width = w;
	m_Height = h;
	
	double aspect = (double)m_Height/m_Width;
	
	m_Left = -1;
	m_Right = 1;
	m_Bottom = -aspect;
	m_Top = aspect;
	
	glViewport(0,0,w,h);
	
	SetViewport(-1,1,-aspect,aspect);
}

void TestBed::Display(void)
{
	glClear(GL_COLOR_BUFFER_BIT);
	glLoadIdentity();
		
	DisplayFPS();			
	
	glTranslatef(-m_CenterX,-m_CenterY,0);
	
	Render();
	
	glutSwapBuffers();
}

void TestBed::Render(void)
{
	glBegin(GL_QUADS);
		glColor3f(0.6,0.6,0);
		glVertex3f(-1,1,0);
		
		glColor3f(0.4,0.4,0.2);
		glVertex3f(1,1,0);
		
		glColor3f(0.2,0.4,0.6);
		glVertex3f(1,-1,0);
		
		glColor3f(0.0,0.6,0.6);
		glVertex3f(-1,-1,0);			
	glEnd();
}

void TestBed::BeginMainLoop(void)
{
	TestBed::m_Instance = this;	
	glutMainLoop();
}

void TestBed::EnableTimer(int t)
{
	m_MSPerFrame = t;
	glutTimerFunc(t,&TestBed::TIMER_CALLBACK,0);
}

void TestBed::Update(void)
{
	// do something!
}

void TestBed::SetViewport(double left, double right, double bottom, double top)
{
	m_Left = left;
	m_Right = right;
	m_Bottom = bottom;
	m_Top = top;
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(left,right,bottom,top); 
	
	glMatrixMode(GL_MODELVIEW);
}

void TestBed::SetCenter(double x, double y)
{
	m_CenterX = x;
	m_CenterY = y;
}

void TestBed::KeyListener(unsigned char key, int x, int y)
{
	if (key==' ') m_IsUpdating = !m_IsUpdating;
}

int TestBed::Time()
{
	return glutGet(GLUT_ELAPSED_TIME);
}

void TestBed::PrintString(char* s)
{
	static void* font = {GLUT_BITMAP_9_BY_15};

	if (s && std::strlen(s))
	{
		while (*s)
		{
			glutBitmapCharacter((void*)font,*s);
			s++;
		}
	}
}

void TestBed::DisplayFPS()
{
	glColor3f(1,1,1);
	WindowPos2f(m_Width-100,m_Height-15);

	char buffer[10];
	std::sprintf(buffer,"fps: %g",m_FPS);	
	PrintString(buffer);
	
	WindowPos2f(m_Width-109,m_Height-30);
	double mspf = 1000*1/m_FPS; //milliseconds per frame
	std::sprintf(buffer,"mspf: %g",mspf);
	PrintString(buffer);
		
	WindowPos2f(m_Width-109,m_Height-45);
	std::sprintf(buffer,"msps: %g",mspf- m_MSPerFrame);
	PrintString(buffer);
}

void TestBed::WindowPos2f( GLfloat x, GLfloat y)
{	
	double z = 0;
	double w = 1;

   GLfloat fx, fy;

   /* Push current matrix mode and viewport attributes */
   glPushAttrib( GL_TRANSFORM_BIT | GL_VIEWPORT_BIT );

   /* Setup projection parameters */
   glMatrixMode( GL_PROJECTION );
   glPushMatrix();
   glLoadIdentity();
   glMatrixMode( GL_MODELVIEW );
   glPushMatrix();
   glLoadIdentity();

   glDepthRange( z, z );
   glViewport( (int) x - 1, (int) y - 1, 2, 2 );

   /* set the raster (window) position */
   fx = x - (int) x;
   fy = y - (int) y;
   glRasterPos4f( fx, fy, 0.0, w );

   /* restore matrices, viewport and matrix mode */
   glPopMatrix();
   glMatrixMode( GL_PROJECTION );
   glPopMatrix();

   glPopAttrib();
}
