
#include "testbed3d.h"
#include <cstring>
#include <cstdio>
#include <cmath>

TestBed3d::TestBed3d(int argc, char** argv): TestBed(argc,argv), m_Camera(Vector3(0,1,5)), m_LookAt(Vector3::ZERO)
{	
	Setup();
}

TestBed3d::TestBed3d(int argc, char** argv,int w, int h): TestBed(argc,argv,w,h), m_Camera(Vector3(0,1,5)), m_LookAt(Vector3::ZERO)
{
	Setup();
}

TestBed3d::~TestBed3d(){}

void TestBed3d::GLSetup()
{
	glClearColor(0,0,0,0);
	
	double aspect = (double)m_Height/m_Width;
	
	Set3dViewport(aspect);
}

void TestBed3d::Reshape(int w, int h)
{
	m_Width = w;
	m_Height = h;
	
	double aspect = (double)m_Height/m_Width;
		
	glViewport(0,0,w,h);
	
	Set3dViewport(aspect);
}

void TestBed3d::Display(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	
	bool resetLight = false;
	
	if (glIsEnabled(GL_LIGHTING))
	{
		resetLight = true;
		glDisable(GL_LIGHTING);
	}

	glDisable(GL_DEPTH_TEST);
	
	DisplayFPS();	
	
	glEnable(GL_DEPTH_TEST);
	
		gluLookAt(m_Camera[0],m_Camera[1],m_Camera[2],m_LookAt[0],m_LookAt[1],m_LookAt[2],0,1,0);
		
	RenderGuides();
	
	if (resetLight)
	{
		glEnable(GL_LIGHTING);
	}	
	
	Render();
	
	glutSwapBuffers();
}

void TestBed3d::Set3dViewport(double aspect)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60,1/aspect,.1,1000); 
	
	glMatrixMode(GL_MODELVIEW);
}

void TestBed3d::RotateAroundY(double angle)
{
	Vector3 dV = m_Camera - m_LookAt;
	
	double x = dV[0];
	double z = dV[2];
	
	x = dV[0]*cos(angle) + dV[2]*sin(angle);
	z = -dV[0]*sin(angle) + dV[2]*cos(angle);
	
	dV[0] = x;
	dV[2] = z;
	
	m_Camera = dV + m_LookAt;	
}

void TestBed3d::RenderGuides()
{
	glColor3f(0.5,0.5,0.5);
	
	glBegin(GL_LINE_LOOP);
		glVertex3f(-1,-1,-1);
		glVertex3f(1,-1,-1);
		glVertex3f(1,-1,1);
		glVertex3f(-1,-1,1);
	glEnd();
	
	glBegin(GL_LINE_LOOP);
		glVertex3f(-1,1,-1);
		glVertex3f(1,1,-1);
		glVertex3f(1,1,1);
		glVertex3f(-1,1,1);
	glEnd();
	
	glBegin(GL_LINES);
		glVertex3f(-1,-1,-1);
		glVertex3f(-1,1,-1);
		
		glVertex3f(1,-1,-1);
		glVertex3f(1,1,-1);
		
		glVertex3f(1,-1,1);
		glVertex3f(1,1,1);
		
		glVertex3f(-1,-1,1);
		glVertex3f(-1,1,1);		
	glEnd();

}

void TestBed3d::EnableLighting()
{

	GLfloat col[] = {1,1,1,1.0};
	
	GLfloat spec[] = {.1,.1,.1,1.0};
	GLfloat shine[] = {1.0};
	
	GLfloat pos[] = {-0.1,5,10,1};
	
	GLfloat diff[] = {1.0,1.0,1.0,1.0};

	GLfloat lmodel_ambient[] = {0.2,0.2,0.2,1};
	
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, col);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, spec);	
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shine);	
		
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diff);
	glLightfv(GL_LIGHT0, GL_SPECULAR, diff);
	glLightfv(GL_LIGHT0, GL_POSITION, pos);
		
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
	
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_COLOR_MATERIAL);
}

void TestBed3d::Zoom(double amount)
{
	Vector3 dV = m_Camera - m_LookAt;
	dV = dV/amount;
	m_Camera = m_LookAt + dV;	
}
