/*
 * Tests: PSystem, World, Particle and Force
 * 
 */
 
#include "PSystem.h"
#include "testbed.h"
#include "RigidStrut.h"
#include "MPWorld.h"
#include "Collision.h"
#include <ctime>

#include <iostream>

using namespace Penguin;

const int NUMPARTICLES = NP; //DEFINED WHEN COMPILED

double w = 2;		

class TestPSystem: public TestBed
{
	PSystem m_System;
	
	ColSphere* col[NUMPARTICLES];
	
	Vector3 colours[NUMPARTICLES];
	
	public:
	TestPSystem(int argc,char**argv);
	virtual void Render();
	virtual void Update();
		
	void DrawParticles();	
};

void TestPSystem::DrawParticles()
{
	int i;
	for (i=0; i<NUMPARTICLES; i++)
	{	
		glColor3f(colours[i][0],colours[i][1],colours[i][2]);

		Vector3& pos = col[i]->GetTarget()->GetPos();

		glPushMatrix();
		glTranslatef(pos[0],pos[1],pos[2]);
		glutSolidSphere(col[i]->GetRadius(),16,16);	
		glPopMatrix();		
	}
	
	glColor3f(1,1,1);
	glBegin(GL_LINE_LOOP);
		glVertex3f(-w,-w,0);
		glVertex3f(w,-w,0);
		glVertex3f(w,w,0);
		glVertex3f(-w,w,0);
	glEnd();
}

int main(int argc, char** argv)
{
	srand(time(NULL));
	TestPSystem test(argc,argv);
	test.EnableTimer(10); // enable update every 20 milliseconds
	test.BeginMainLoop();
	
	return 0;
}
double randomDBL()
{
        return double(std::rand())/RAND_MAX;
}

TestPSystem::TestPSystem(int argc,char**argv):TestBed(argc,argv)
{
	Setup();	

	World* world = m_System.AddWorld("Test", new World(5,0.05));
	if (!world)
	{
		std::cout << "Can't create world!" << std::endl;
	}
	else
	{
		Object* tempObj = new Object();
		world->ManageObject(tempObj,"moving");
		
		Object* staticObj = new Object();
		staticObj->SetInternalCollisions(false);
		world->ManageObject(staticObj,"unmoving");				
		
		
		ColPlane* cp1 = new ColPlane(Vector3(0,1,0),Vector3(0,-w,0));
		ColPlane* cp2 = new ColPlane(Vector3(1,0,0),Vector3(-w,0,0));
		ColPlane* cp3 = new ColPlane(Vector3(0,-1,0),Vector3(0,w,0));
		ColPlane* cp4 = new ColPlane(Vector3(-1,0,0),Vector3(w,0,0));
		
		staticObj->AddCollisionStruct(cp1);
		staticObj->AddCollisionStruct(cp2);
		staticObj->AddCollisionStruct(cp3);
		staticObj->AddCollisionStruct(cp4);
		
		int i;
		double size;		
		for (i = 0; i < NUMPARTICLES; ++i)
		{
			size = 0.1*(randomDBL()+1);
			
			Particle* p = new Particle();
			
			p->SetVel(Vector3(1-2*randomDBL(),1-2*randomDBL(),0));
						
			col[i] = new ColSphere(p,size);
			col[i]->SetCoefficientOfRestitution(BNC); //DEFINED WHEN COMPILED
			/* let tempObj keep track of the particle */
			tempObj->AddParticle(p);
			tempObj->AddCollisionStruct(col[i]);			
						
			bool hasCollided = true;
			
			int tries = 0;
			
			while (hasCollided)
			{
				hasCollided = false;
				
				col[i]->GetTarget()->SetPos(Vector3(1-2*randomDBL(),1-2*randomDBL(),0)*0.9*w);
				int j;
				for (j=0; j<i; j++)
				{
					if ((col[i]->GetTarget()->GetPos()-col[j]->GetTarget()->GetPos()).GetLength() < (col[i]->GetRadius()+col[j]->GetRadius()))
					{
						hasCollided = true;
						break;
					}
				}
				
				tries++;
				
				if (tries>10000)
				{
					std::cerr << "Too many particles to fit inside the box!\n";
					exit(-1);
				}
			}
						
			colours[i] = Vector3(randomDBL(),randomDBL(),randomDBL());
		}	
		m_IsUpdating = false;
	}	
}

void TestPSystem::Render(void)
{		
	SetViewport(-4,4,-4,4);
	DrawParticles();	
}

void TestPSystem::Update(void)
{
	static double dt = m_MSPerFrame/1000.0;	
	m_System.Simulate(dt);
}


