#ifndef FORCE_H
#define FORCE_H

#include "Particle.h"

namespace Penguin
{
	/*! \brief An abstract interface for all Force Objects.
	 *
	 * A Force Object is an object that directly manipulates a particle's force accumulator.
	 *
	 * Each new Force class <b>must</b> satisfy this interface.
	 */	 
	class Force
	{
		public:		
		/*! Each Force class must define this method. */
		virtual void ApplyForce(void) = 0;
	};
	
	/*! \brief A simple linear impulse force.
	*
	* Applies a simple force vector on a certain particle.
	*/
	class ImpulseForce
	{
		Particle* m_Target;
		Vector3 m_Force;		
	
		public:
		
		ImpulseForce(Particle* target, Vector3 force)
		{
			m_Target = target;
			m_Force = force;
		}
		
		/*! Returns the target of the impulse force */
		Particle* GetTarget(){return m_Target;}
		/*! Returns the impulse force vector */
		Vector3& GetForce(){return m_Force;}
		
		/*! Decreases the force vector by an amount <b>factor</b>. E.g. <b>factor=2</b> will halve the impulse force. */
		void DecreaseForce(REAL factor)
		{
			m_Force /= factor;
		}
		
		virtual void ApplyForce(void)
		{
			m_Target->GetForce()+=m_Force;
		}
	};
}

#endif
