#ifndef PARAMETRICPARTICLE_H
#define PARAMETRICPARTICLE_H

#include <cmath>

namespace Penguin
{
	
	/*! \brief This Parametric Particle doesn't move. */
	class StableParticle: public Particle
	{
		public:
		StableParticle(Vector3 pos){SetPos(pos);}

		virtual void Simulate(double t){}
	};
	
	/*! \brief This Parametric Particles follows a circular path. */
	class CircleParticle: public Particle
	{
		double r;
		double s; //revs per second

		Vector3 center;

		public:
		/*! Specify the centroid, radius and revolutions per second. */
		CircleParticle(Vector3 pos, double radius=1.0, double speed=1.0)
		{
			r = radius;
			s = speed;
			center = pos;
		}

		virtual void Simulate(double t)
		{	
			Vector3 rel(sin(s*t),cos(s*t),0);
			SetPos(center + rel*r);
		}
	};
	
	
}

#endif
