#include "Collision.h"

namespace Penguin
{
	//Detects Collision between two spheres
	CollisionInfo DetectCollision(ColSphere* s1, ColSphere* s2, double dt)
	{
		/* First perform a quick static test */
		
		//vector from sphere2 to sphere1
		Vector3 vectorFromP2ToP1 = s1->GetTarget()->GetPos() - s2->GetTarget()->GetPos();
		
		//minimum distance before collision
		REAL combinedRadii = fabs(s1->GetRadius() + s2->GetRadius());
		REAL length = vectorFromP2ToP1.GetLength();
		
		REAL dist = length - combinedRadii;
		bool areIntersecting = dist<0;
				
		CollisionInfo ci;
		ci.areIntersecting = areIntersecting;	
		ci.type1 = COL_SPHERE;
		ci.type2 = COL_SPHERE;
		ci.target1 = s1;
		ci.target2 = s2;
		ci.relativeVelocity = s1->GetTarget()->GetVel() - s2->GetTarget()->GetVel();
		
		if (!areIntersecting)
		{
			return ci; //unimplemented yet			
			//Do a dynamic test.
			
			//Take s1 to be static and s2 to be moving.
			//Thus we are intersecting a sphere and a capsule
			/*			
			Vector3& center1 = s1->GetTarget()->GetPos();
			Vector3& relvel = ci.relativeVelocity;
			Vector3 ptB = s2->GetTarget()->GetPos();
			Vector3 ptA = ptB - relvel*dt;
			*/			
		}
		else
		{
			//Do a static test 
			ci.collisionNormal = vectorFromP2ToP1/length;
			ci.velocityProjectionOnDist = ci.relativeVelocity.Dot(ci.collisionNormal);
			ci.intersectionDistance = -dist;
			return ci;
		}
	}
	
	//Detect Collision between a plane and a sphere
	CollisionInfo DetectCollision(ColPlane* p1, ColSphere* s2)
	{
		//Vector from planeCentroid to Sphere
		Vector3 planeToSphere = s2->GetTarget()->GetPos() - p1->GetPos();
		
		//Distance from plane to sphere
		REAL distance = planeToSphere.Dot(p1->GetNormal());
		
		CollisionInfo ci;
		ci.areIntersecting = distance < s2->GetRadius();
		
		ci.type1 = COL_PLANE;
		ci.type2 = COL_SPHERE;
		
		ci.target1 = p1;
		ci.target2 = s2;
		
		ci.relativeVelocity = s2->GetTarget()->GetVel();
		ci.velocityProjectionOnDist = planeToSphere.Dot(ci.relativeVelocity);
		
		ci.collisionNormal = p1->GetNormal();
		ci.intersectionDistance = (s2->GetRadius() - distance);
				
		return ci;
	}

	CollisionInfo DetectCollision(ColSphere* s, ColOrientedFace* o)
	{
		CollisionInfo ci;
		
		// Take p to be the center pt of the oriented face
		Vector3 p = o->GetPos();

		// Vector from p to s, and the distance between the plane and sphere
		Vector3 ps = s->GetTarget()->GetPos() - p;
		REAL distance = ps.Dot(o->GetNormal());
		
		if (fabs(distance) < s->GetRadius()) //Then collision is occurring
		{
			Vector3 sideways = o->GetTop().Cross(o->GetNormal()).Normal();
			Vector3& top = o->GetTop();
		
			if (	(fabs(ps.Dot(top)) < (o->GetHeight()/2+s->GetRadius())) // If it is within the top vector direction
			&&	((fabs(ps.Dot(sideways))) < (o->GetWidth()/2+s->GetRadius()))) // And within the side vector direction
			{
				//Collision has occurred
				ci.areIntersecting = true;
		
				//Hmmm...why do we have to do this when areIntersecting is false?
				ci.type2 = COL_ORFACE;
				ci.type1 = COL_SPHERE;
		
				ci.target2 = o;
				ci.target1 = s;
		
				ci.relativeVelocity = s->GetTarget()->GetVel();
				ci.velocityProjectionOnDist = ps.Dot(ci.relativeVelocity);		
				ci.collisionNormal = o->GetNormal();
				//Ignore total penetration?
				ci.intersectionDistance = (s->GetRadius() - distance);		
				return ci;				
			}			
		}
	
		ci.areIntersecting = false;
		return ci;
	};

//Detect Collision between a sphere and an oriented face
/*	CollisionInfo DetectCollision(ColSphere* s, ColOrientedFace* o)
	{
		//Vector from planeCentroid to Sphere
		Vector3 planeToSphere = s->GetTarget()->GetPos() - o->GetPos();
		
		//Distance from plane to sphere
		REAL distance = planeToSphere.Dot(o->GetNormal());
		
		CollisionInfo ci;

		//Whether the sphere and the face are on the same plane
		bool samePlane = distance < s->GetRadius();

		if (!samePlane) ci.areIntersecting = false;
		else 
		{
			//top is not just the direction, it's the top/bottom edge
			Vector3 top = o->GetTop();
			top = top * o->GetWidth();

			if (planeToSphere.Dot(top) > 0)
			{
				//side is not just the direction, it's the lef/right edge
				//Watch out for the right-hand rule the following is not correct (?)
				//Vector3 side = o->GetNormal().Cross(o->GetTop());
				Vector3 side = o->GetTop().Cross(o->GetNormal());
				side = side * o->GetHeight();
	
				if (planeToSphere.Dot(side) > 0)
				{
					Vector3 origin = o->GetPos();
					Vector3 midRight = origin + top + (side*0.5);
					Vector3 midBottom = origin + side + (top*0.5);
					Vector3 sphereLocation = s->GetTarget()->GetPos();
					Vector3 rightVector = sphereLocation - midRight;
					Vector3 bottomVector = sphereLocation - midBottom;
					REAL rightDotProduct = rightVector.Dot(top);
					REAL bottomDotProduct = bottomVector.Dot(side);

					//Sphere inside face
					if (rightDotProduct < 0 && bottomDotProduct < 0) ci.areIntersecting = true;
					else ci.areIntersecting = false;
				}
				else ci.areIntersecting = false;
			}		
			else ci.areIntersecting = false;
		}
		
		//Hmmm...why do we have to do this when areIntersecting is false?
		ci.type2 = COL_ORFACE;
		ci.type1 = COL_SPHERE;
		
		ci.target2 = o;
		ci.target1 = s;
		
		ci.relativeVelocity = s->GetTarget()->GetVel();
		ci.velocityProjectionOnDist = planeToSphere.Dot(ci.relativeVelocity);
		
		ci.collisionNormal = o->GetNormal();
		//Ignore total penetration?
		ci.intersectionDistance = (s->GetRadius() - distance);
		
		return ci;
	}
	*/
	//Detect Collision between a sphere and a cylinder
	CollisionInfo DetectCollision(ColSphere* s, ColCylinder* c)
	{
		Vector3 top = c->GetTop();
		Vector3 bottom = c->GetBottom();
		REAL c_radius = c->GetRadius();
		Vector3 sphereLocation = s->GetTarget()->GetPos();
		REAL s_radius = s->GetRadius();

		Vector3 top2Sphere = sphereLocation - top;
		Vector3 bottom2Sphere = sphereLocation - bottom;
		Vector3 bottom2Top = top - bottom;
		//Distance from sphere to the top surface
		REAL d1 = top2Sphere.Dot(bottom2Top);
		//Distance from sphere to the bottom surface
		REAL d2 = bottom2Sphere.Dot(bottom2Top);

		CollisionInfo ci;
		ci.relativeVelocity = s->GetTarget()->GetVel();

		//Case1: sphere is closer the either of the head	
		//d1 and d2 has the same sign
		if (d1*d2 > 0)
		{
			REAL distance;
			if (fabs(d1) < fabs(d2)) 
			{
				distance = d1;
				ci.velocityProjectionOnDist = top2Sphere.Dot(ci.relativeVelocity);
				ci.collisionNormal = bottom2Top.Normal();
			}
			else
			{
				distance = d2;
				ci.velocityProjectionOnDist = bottom2Sphere.Dot(ci.relativeVelocity);
				ci.collisionNormal = (-bottom2Top).Normal();
			}
			ci.areIntersecting = distance < s_radius;
			ci.intersectionDistance = (s_radius - distance);
		}
		//Case2: sphere is closer to the body
		//d1 and d2 has different signs
		else
		{
			//Distance from top to sphere (square)
			REAL sqr_d = top2Sphere.GetSqLength();
			//Can be calculated with either d1 or d2
			REAL sqr_d1 = d1*d1;
			//Distance from sphere to the centre axis of the cylinder
			REAL sphere2Axis = sqrt(sqr_d - sqr_d1);	
			//Determine collision, note: ignore total penetration

			ci.intersectionDistance = s_radius - fabs(sphere2Axis - c_radius);
		
			ci.areIntersecting = ci.intersectionDistance > 0;
			//Can be calculated with either top2Sphere or bottom2Sphere
			ci.velocityProjectionOnDist = top2Sphere.Dot(ci.relativeVelocity);

			//Calculate the collision normal vector, can be done with respect
			//to either the top or the bottom (chosen: the top)
			//s_prj is the projection of the sphere's location on the axis at the centre of the cylinder
			Vector3 s_prj = top - ( bottom2Top.Normal() * fabs(d1) );
			ci.collisionNormal = (sphereLocation - s_prj).Normal();
		}

		ci.type2 = COL_CYLINDER;
		ci.type1 = COL_SPHERE;
		
		ci.target2 = c;
		ci.target1 = s;
		
		return ci;
	}

	//Detect Collision between a sphere and an AABB
	CollisionInfo DetectCollision(ColSphere* s, ColAABB* a)
	{
		Vector3 s_pos = s->GetTarget()->GetPos();
		REAL s_radius = s->GetRadius();
		Vector3 a_centre = a->GetCentre();
		REAL width = a->GetWidth();
		REAL height = a->GetHeight();
		REAL depth = a->GetDepth();

		Vector3 x_axis = Vector3::GetX_axis();
		Vector3 y_axis = Vector3::GetY_axis();
		Vector3 z_axis = Vector3::GetZ_axis();
		
		Vector3 top_centre = a_centre + y_axis*(0.5*height);
		Vector3 bottom_centre = top_centre - (y_axis*height);
		Vector3 right_centre = a_centre + x_axis*(0.5*width);
		Vector3 left_centre = right_centre - (x_axis*width);
		Vector3 front_centre = a_centre + z_axis*(0.5*depth);
		Vector3 back_centre = front_centre - (z_axis*depth);

		CollisionInfo ci;
		ci.relativeVelocity = s->GetTarget()->GetVel();
		
		ci.type1 = COL_SPHERE;
		ci.type2 = COL_AABB;
		
		ci.target1 = s;
		ci.target2 = a;

		ci.areIntersecting = false;

		//Calculate the distance between the sphere and each face
		//to find if the sphere intersects with any
		
		if ( fabs((s_pos-top_centre).Dot(y_axis)) < s_radius )
		{
			REAL x = s_pos.GetX();
			REAL z = s_pos.GetZ();
			Vector3 left = top_centre - x_axis*(0.5 * width);
			Vector3 right = left + x_axis*width;
			REAL top = left.GetZ() - 0.5*depth;
			REAL bottom = top + depth;
			if (x >= left.GetX() && x <= right.GetX() && z >= top && z <= bottom)
			{
				ci.areIntersecting = true;
				ci.collisionNormal = y_axis;
				ci.intersectionDistance = s_radius - ((s_pos-top_centre).Dot(ci.collisionNormal));
				ci.velocityProjectionOnDist = ci.relativeVelocity.Dot(ci.collisionNormal);
			}
		}

		else if ( fabs((s_pos-bottom_centre).Dot(y_axis)) < s_radius )
		{
			REAL x = s_pos.GetX();
			REAL z = s_pos.GetZ();
			Vector3 left = bottom_centre + x_axis*(0.5 * width);
			Vector3 right = left - x_axis*width;
			REAL top = left.GetZ() - 0.5*depth;
			REAL bottom = top + depth;
			if (x <= left.GetX() && x >= right.GetX() && z >= top && z <= bottom)
			{
				ci.areIntersecting = true;
				ci.collisionNormal = -y_axis;
				ci.intersectionDistance = s_radius - ((s_pos-bottom_centre).Dot(ci.collisionNormal));
				ci.velocityProjectionOnDist = ci.relativeVelocity.Dot(ci.collisionNormal);
			}
		}


		else if ( fabs((s_pos-right_centre).Dot(x_axis)) < s_radius )
		{
			REAL y = s_pos.GetY();
			REAL z = s_pos.GetZ();
			Vector3 left = right_centre + z_axis*(0.5 * depth);
			Vector3 right = left - z_axis*depth;
			REAL top = left.GetY() + 0.5*height;
			REAL bottom = top - height;
			if (z <= left.GetZ() && z >= right.GetZ() && y <= top && y >= bottom)
			{
				ci.areIntersecting = true;
				ci.collisionNormal = x_axis;
				ci.intersectionDistance = s_radius - ((s_pos-right_centre).Dot(ci.collisionNormal));
				ci.velocityProjectionOnDist = ci.relativeVelocity.Dot(ci.collisionNormal);
			}
		}


		else if ( fabs((s_pos-left_centre).Dot(x_axis)) < s_radius )
		{
			REAL y = s_pos.GetY();
			REAL z = s_pos.GetZ();
			Vector3 left = left_centre - z_axis*(0.5 * depth);
			Vector3 right = left + z_axis*depth;
			REAL top = left.GetY() + 0.5*height;
			REAL bottom = top - height;
			if (z >= left.GetZ() && z <= right.GetZ() && y <= top && y >= bottom)
			{
				ci.areIntersecting = true;
				ci.collisionNormal = -x_axis;
				ci.intersectionDistance = s_radius - ((s_pos-left_centre).Dot(ci.collisionNormal));
				ci.velocityProjectionOnDist = ci.relativeVelocity.Dot(ci.collisionNormal);
			}
		}


		else if ( fabs((s_pos-front_centre).Dot(z_axis)) < s_radius )
		{
			REAL x = s_pos.GetX();
			REAL y = s_pos.GetY();
			Vector3 left = front_centre - x_axis*(0.5 * width);
			Vector3 right = left + x_axis*width;
			REAL top = left.GetY() + 0.5*height;
			REAL bottom = top - height;
			if (x >= left.GetX() && x <= right.GetX() && y <= top && y >= bottom)
			{
				ci.areIntersecting = true;
				ci.collisionNormal = z_axis;
				ci.intersectionDistance = s_radius - ((s_pos-front_centre).Dot(ci.collisionNormal));
				ci.velocityProjectionOnDist = ci.relativeVelocity.Dot(ci.collisionNormal);
			}
		}


		else if ( fabs((s_pos-back_centre).Dot(z_axis)) < s_radius )
		{
			REAL x = s_pos.GetX();
			REAL y = s_pos.GetY();
			Vector3 left = back_centre + x_axis*(0.5 * width);
			Vector3 right = left - x_axis*width;
			REAL top = left.GetY() + 0.5*height;
			REAL bottom = top - height;
			if (x <= left.GetX() && x >= right.GetX() && y <= top && y >= bottom)
			{
				ci.areIntersecting = true;
				ci.collisionNormal = -z_axis;
				ci.intersectionDistance = s_radius - ((s_pos-back_centre).Dot(ci.collisionNormal));
				ci.velocityProjectionOnDist = ci.relativeVelocity.Dot(ci.collisionNormal);
			}
		}

		return ci;
	}

	bool Overlap(ColAABB* a1, ColAABB* a2)
	{
		Vector3 c1 = a1->GetCentre();
		REAL w1 = a1->GetWidth();
		REAL h1 = a1->GetHeight();
		REAL d1 = a1->GetDepth();
		REAL x1 = c1.GetX();
		REAL y1 = c1.GetY();
		REAL z1 = c1.GetZ();

		Vector3 c2 = a2->GetCentre();
		REAL w2 = a2->GetWidth();
		REAL h2 = a2->GetHeight();
		REAL d2 = a2->GetDepth();
		REAL x2 = c2.GetX();
		REAL y2 = c2.GetY();
		REAL z2 = c2.GetZ();
		
	
		//Check in the x-axis
		bool con_x = ((x1+0.5*w1) > (x2-0.5*w2)) && ((x1-0.5*w1) < (x2+0.5*w2));
		bool con_y = ((y1+0.5*h1) > (y2-0.5*h2)) && ((y1-0.5*h1) < (y2+0.5*h2));
		bool con_z = ((z1+0.5*d1) > (z2-0.5*d2)) && ((z1-0.5*d1) < (z2+0.5*d2));
		
		return (con_x || con_y || con_z);
	}

	void ColAABB::InitPoints()
	{
		Vector3 x_axis = Vector3::GetX_axis();
		Vector3 y_axis = Vector3::GetY_axis();
		Vector3 z_axis = Vector3::GetZ_axis();

		t1 = m_centre - z_axis*m_depth/2 - x_axis*m_width/2 + y_axis*m_height/2;
		t2 = t1 + x_axis*m_width;
		t3 = t2 + z_axis*m_depth;
		t4 = t1 + z_axis*m_depth;

		b1 = t1 - y_axis*m_height;
		b2 = t2 - y_axis*m_height;
		b3 = t3 - y_axis*m_height;
		b4 = t4 - y_axis*m_height;
	}

	std::vector<Vector3*> ColAABB::GetFrontFace()
	{
		std::vector<Vector3*> vec;
		vec.push_back(&t4);
		vec.push_back(&t3);
		vec.push_back(&b3);
		vec.push_back(&b4);
		return vec;	
	}

        std::vector<Vector3*> ColAABB::GetBackFace()
	{
		std::vector<Vector3*> vec;
		vec.push_back(&t2);
		vec.push_back(&t1);
		vec.push_back(&b1);
		vec.push_back(&b2);
		return vec;	
	}

        std::vector<Vector3*> ColAABB::GetTopFace()
	{
		std::vector<Vector3*> vec;
		vec.push_back(&t1);
		vec.push_back(&t2);
		vec.push_back(&t3);
		vec.push_back(&t4);
		return vec;	
	}

        std::vector<Vector3*> ColAABB::GetBottomFace()
	{
		std::vector<Vector3*> vec;
		vec.push_back(&b1);
		vec.push_back(&b2);
		vec.push_back(&b3);
		vec.push_back(&b4);
		return vec;	
	}

        std::vector<Vector3*> ColAABB::GetLeftFace()
	{
		std::vector<Vector3*> vec;
		vec.push_back(&t1);
		vec.push_back(&t4);
		vec.push_back(&b4);
		vec.push_back(&b1);
		return vec;	
	}
        std::vector<Vector3*> ColAABB::GetRightFace()
	{
		std::vector<Vector3*> vec;
		vec.push_back(&t3);
		vec.push_back(&t2);
		vec.push_back(&b2);
		vec.push_back(&b3);
		return vec;	
	}
}
