#include "DampedSpring.h"
#include <cmath>

namespace Penguin
{
		DampedSpring::DampedSpring(Particle* one, Particle* two, double l, double k, double c)
		:m_One(one),m_Two(two),m_L(l),m_K(k),m_C(c){}
		
		void DampedSpring::ApplyForce(void)
		{ 
			/* 
				F_A = (-k (|L|-R) + c Ldot.L / |L| ) L/|L|
				
				where L = vector between the two particles.
				R = rest length
			*/
			
			Vector3 one_to_two = m_One->GetPos()-m_Two->GetPos();
					
			double dx = one_to_two.GetLength();
			double dist = fabs(dx) - m_L;
			
			REAL spring_force = - m_K * dist;
			
			Vector3 one_to_two_normed = one_to_two / dx;
			
			Vector3 Ldot = m_Two->GetVel()-m_One->GetVel();
			
			REAL damping_force = m_C * Ldot.Dot(one_to_two_normed);
			
			Vector3 F_A = one_to_two_normed * (spring_force + damping_force);
			
			m_One->GetForce() += F_A;
			m_Two->GetForce() -= F_A;
		}
}
