#include "Emitter.h"

/* Emitter implementation */

namespace Penguin
{
	Emitter::Emitter(int max_num = 500, double mass = 1.0):m_EmitterBehaviour(0), m_NumParticles(max_num), m_CurrentParticles(0),m_Rate(0.05),m_TimeSinceLastEmit(0),m_mass(mass)
	{

  }		

	Emitter::~Emitter()
	{
		delete[]m_LifeArray;
		delete m_EmitterBehaviour;
	}

	void Emitter::Setup()
	{
		m_LifeArray = new double[m_NumParticles];

		int i;
		for(i=0; i<m_NumParticles; i++)
		{
			m_LifeArray[i] = -1;
			Particle* p = new Particle();
			p->SetMass(m_mass);
			AddParticle(p);
		}
	}

	void Emitter::Update(double dt)
	{
		/* Update particles */

		UpdateParticles(dt);

		/* Update time and emit particles */

		m_TimeSinceLastEmit += dt;

		int num2emit = (int)(m_TimeSinceLastEmit / m_Rate);

		if (num2emit>0)
		{
			m_TimeSinceLastEmit = 0;

			Emit(num2emit);
		}	
	}

	void Emitter::Emit(int num)
	{
			while(num>0&&m_CurrentParticles<m_NumParticles)
			{
				m_LifeArray[m_CurrentParticles] = m_InitialLife; // reset the life

				//Reset Particle
				m_Particles[m_CurrentParticles]->SetVel(Vector3::ZERO);
				
				//Let the behaviour determine the new properties.
				m_EmitterBehaviour->Emit(m_Particles[m_CurrentParticles]);
				m_CurrentParticles++;
				num--;	
			}
	}

	void Emitter::UpdateParticles(double dt)
	{
		int i;
		for(i=0; i<m_CurrentParticles; i++)
		{
			m_LifeArray[i] -= dt;

			if (m_LifeArray[i]<=0)
			{
				m_LifeArray[i] = m_LifeArray[m_CurrentParticles-1];

				Particle* p = m_Particles[i];
				m_Particles[i] = m_Particles[m_CurrentParticles-1];
				m_Particles[m_CurrentParticles-1] = p;

				m_CurrentParticles--;
				i--;			
			}
		}
	}

	PIter Emitter::GetParticlesEnd()
	{
		return m_Particles.begin()+m_CurrentParticles;
	}
	
	/* Collidable Emitter */
		
	CollidableEmitter::CollidableEmitter(int max_num = 500, double mass = 1.0, REAL radius = 0.025): Emitter(max_num,mass),m_radius(radius)
	{
	
	}
	
	void CollidableEmitter::Setup()
	{
		m_LifeArray = new double[m_NumParticles];

		int i;
		for(i=0; i<m_NumParticles; i++)
		{
			m_LifeArray[i] = -1;
			Particle* p = new Particle();
			ColSphere* cs = new ColSphere(p,m_radius);
			AddParticle(p);
			AddCollisionStruct(cs);
		}
	}

	ColIter CollidableEmitter::GetCollisionStructsEnd()
	{	
		return m_CollisionStructs.begin() + m_CurrentParticles;
	}
	
	void CollidableEmitter::UpdateParticles(double dt)
	{
		int i;
		for(i=0; i<m_CurrentParticles; i++)
		{
			m_LifeArray[i] -= dt;

			if (m_LifeArray[i]<=0)
			{
				m_LifeArray[i] = m_LifeArray[m_CurrentParticles-1];

				Particle* p = m_Particles[i];
				m_Particles[i] = m_Particles[m_CurrentParticles-1];
				m_Particles[m_CurrentParticles-1] = p;

				ColBase* b = m_CollisionStructs[i];
				m_CollisionStructs[i] = m_CollisionStructs[m_CurrentParticles-1];
				m_CollisionStructs[m_CurrentParticles-1] = b;

				m_CurrentParticles--;
				i--;			
			}
		}
	}
} //end namespace Penguin
