#include "MPWorld.h"
#include "PSystem.h"

#include <iostream>
#include <algorithm>

namespace Penguin
{
	void MPWorld::Simulate(double dt)
	{
		//Store old positions (s) and velocities (v)
		
		std::vector<Vector3> old_s;
		std::vector<Vector3> old_v;
		std::vector<Vector3> new_v;
		
		VIter s_it;
		VIter v_it;
	
		ObjIter itm;
		for (itm=m_Objects.begin();itm!=m_Objects.end();++itm)
		{
			PIter it = itm->second->GetParticlesBegin();
			PIter it2 = itm->second->GetParticlesEnd();			
			int sz = it2-it;			
			
			old_s.resize(old_s.size()+sz);
			old_v.resize(old_v.size()+sz);
			
			s_it = old_s.end()-sz;
			v_it = old_v.end()-sz;
			
			for (;it!=it2;++it,++s_it,++v_it)
			{
				(*s_it) = (*it)->GetPos();
				(*v_it) = (*it)->GetVel();
			}
		}
		
		//Take a half Euler step (thus retrieving a new s and v)
		ZeroForces();
		ApplyForces();		
		ApplyInitialImpulses(dt);
		SatisfyConstraints();		
		
		double halfstep = dt/2;

		ApplyEulerStep(halfstep);
		SimulateParametricParticles(m_Parent->GetTime()-halfstep);
		
		//Calculate new forces
		
		ZeroForces();
		ApplyForces();
		ApplyImpulses(1);		
		SatisfyConstraints();
		
		//Rewind and "Remember" new velocity and new forces
		
		s_it = old_s.begin();
		v_it = old_v.begin();
		new_v = old_v;
		
		for (itm=m_Objects.begin();itm!=m_Objects.end();++itm)
		{
			PIter it = itm->second->GetParticlesBegin();
			PIter it2 = itm->second->GetParticlesEnd();			
			
			for (;it!=it2;++it,++s_it,++v_it)
			{
				(*it)->GetPos() = (*s_it);
				(*it)->GetVel() = (*v_it);
			}
		}
		
		//Take a full Euler Step using the old s,v.
	
		v_it = new_v.begin();
		
		for(itm=m_Objects.begin();itm!=m_Objects.end();++itm)
		{
			PIter it = itm->second->GetParticlesBegin();
			PIter it2 = itm->second->GetParticlesEnd();
						
			for (;it!=it2;++it,++v_it)
			{				
				Particle* p = (*it);				
				p->GetVel() += p->GetForce()*dt/p->GetMass();
				p->GetPos() += (*v_it)*dt;
			}			
		}
		
		SimulateParametricParticles(m_Parent->GetTime());
		ApplyCollision(dt);		
		UpdateObjects(dt);		
		
		//END

	}
	
	void MPWorld::ApplyInitialImpulses(double dt)
	{
		ImpIter it;
		for(it=m_ImpulseVector.begin();it!=m_ImpulseVector.end();++it)
		{
			it->DecreaseForce(dt);
			it->ApplyForce();			
			it->DecreaseForce(2);
		}		
	}
}
