#include "Object.h"

namespace Penguin
{
	Object::~Object()
	{
		PIter it = m_Particles.begin();
		PIter it2 = m_Particles.end();
		
		while(it!=it2)
		{
			if (*it) delete (*it);
			++it;
		}
		
		it = m_ParametricParticles.begin();
		it2 = m_ParametricParticles.end();
		
		while(it!=it2)
		{
			if (*it) delete (*it);
			++it;
		}
		
		ColIter cit = m_CollisionStructs.begin();
		ColIter cend = m_CollisionStructs.end();
		for(;cit!=cend;++cit)
		{
			if (*cit) delete (*cit);
		}
	}

	Particle* Object::AddParticle(Particle* p)
	{
		if (p==NULL) return NULL;
		else m_Particles.push_back(p);
		return p;
	}
						
	/*! Add a parametric particle to the simulation. */
	Particle* Object::AddParametricParticle(Particle* p)
	{
		if (p==NULL) return NULL;
		else m_ParametricParticles.push_back(p);
		return p;
	}

	/*! Add a collidable shape/structure to the simulation - DuCK. */
  ColBase* Object::AddCollisionStruct(ColBase* p)
  {
  	if (p==NULL) return NULL;
		else m_CollisionStructs.push_back(p);
		return p;
  }
		
	/*! Add a new force to the system. */
	Force* Object::AddForce(Force* f)
	{
		if (f==NULL) return f;
		else m_Forces.push_back(f);
		return f;
	}
						
	/*! Add a new constraint to the system. */
	Constraint* Object::AddConstraint(Constraint* f)
	{
		if (f==NULL) return f;
		else m_Constraints.push_back(f);
		return f;	
	}
		
	void Object::SatisfyConstraints()
	{
		ConsIter it = m_Constraints.begin();
		ConsIter end = m_Constraints.end();
		while (it!=end)
		{
			(*it)->Apply();
			it++;
		}
	}
	
	void Object::ApplyForces()
	{
		FIter it = m_Forces.begin();
		FIter end = m_Forces.end();
		while (it!=end)
		{
			(*it)->ApplyForce();
			it++;
		}		
	}
	
	void Object::SimulateParametricParticles(double t)
	{
		PIter it = m_ParametricParticles.begin();
		PIter end = m_ParametricParticles.end();
		while (it!=end)
		{
			(*it)->Simulate(t);
			it++;
		}
	}		
}
