#include "PSystem.h"

namespace Penguin
{

	PSystem::PSystem()
	{
		m_Time = 0;
	}

	PSystem::~PSystem()
	{
		std::map<std::string,World*>::iterator it = m_Worlds.begin();
		for (;it!=m_Worlds.end();++it)
		{
			if (it->second) delete (it->second);
		}
	}

	World* PSystem::AddWorld(std::string name, World* world)
	{	
		if (m_Worlds.find(name)==m_Worlds.end())
		{
			m_Worlds[name] = world;
			world->SetParent(this);
			return world;
		}
		else
		{
			delete world;
			return NULL;
		}
	}

	World* PSystem::GetWorld(std::string name)
	{
		std::map<std::string,World*>::iterator it = m_Worlds.find(name);
		if (it==m_Worlds.end()) return NULL;
		else return it->second;
	}

	World* PSystem::operator[](std::string name)
	{
		std::map<std::string,World*>::iterator it = m_Worlds.find(name);
		if (it==m_Worlds.end()) return NULL;
		else return it->second;
	}

	void PSystem::RemoveWorld(std::string name)
	{
		std::map<std::string,World*>::iterator it = m_Worlds.find(name);
		if (it!=m_Worlds.end()) m_Worlds.erase(it);
	}

	void PSystem::Simulate(double dt)
	{
		std::map<std::string,World*>::iterator it = m_Worlds.begin();
		for (;it!=m_Worlds.end();++it)
		{
			it->second->Simulate(dt);
		}

		m_Time += dt;
	}

	double PSystem::GetTime()
	{
		return m_Time;
	}
}
