#include "RigidStrut.h"

#include <iostream>


Penguin::RigidStrut::RigidStrut(Particle* one, Particle* two, double k, double c)
:m_One(one),m_Two(two),m_K(k),m_C(c)
{
	/* maybe do a quick check for satisfiability...?*/
	
	m_SquareRootW = m_Two->GetMass()/m_One->GetMass();
	m_W = m_SquareRootW*m_SquareRootW;
	m_L2 = (m_One->GetPos() - m_Two->GetPos()).GetSqLength();		
}

void Penguin::RigidStrut::Apply()
{
	Particle* a = m_One;
	Particle* b = m_Two;

	/* d = a-b, dd = d' = a' - b' */

	Vector3 d = a->GetPos()-b->GetPos();
	Vector3 dd = a->GetVel()-b->GetVel();

	double d2 = d.GetSqLength();

	/* C = 0.5 d*d - 0.5 L^2, C' = CC = d.d'*/
	
	double C = 0.5 * (d2 - m_L2);
	double CC = d.Dot(dd);

	double& ma = a->GetMass();
	double& mb = b->GetMass();

	double J = ma*mb*(dd.GetSqLength() - m_K*C - m_C*CC) + 
							ma*(d.Dot(b->GetForce())) - mb*(d.Dot(a->GetForce()));
	J /= mb * d2;
	
	/* Final constraint equations */
	
	double lambda_a = J*m_W/(1+m_W);
	double lambda_b = (lambda_a - J)*m_SquareRootW;
	
	a->GetForce() += d*lambda_a;
	b->GetForce() += d*lambda_b;	
}
