#include "Vector3.h"
#include <cmath>


namespace Penguin
{
	const Vector3 Vector3::ZERO = Vector3();

	Vector3 Vector3::GetX_axis()
	{
		return Vector3(1.0, 0.0, 0.0);
	}

	Vector3 Vector3::GetY_axis()
	{
		return Vector3(0.0, 1.0, 0.0);
	}

	Vector3 Vector3::GetZ_axis()
	{
		return Vector3(0.0, 0.0, 1.0);
	}

	Vector3::Vector3()
	{
		m_data[0] = m_data[1] = m_data[2] = 0;
	}
	
	Vector3::Vector3(REAL x, REAL y, REAL z)
	{
		m_data[0] = x;
		m_data[1] = y;
		m_data[2] = z;
	}
	
	Vector3::Vector3(const Vector3& v)
	{
		m_data[0] = v.m_data[0];
		m_data[1] = v.m_data[1];
		m_data[2] = v.m_data[2];
	}
	
	REAL& Vector3::operator[](int index)
	{
		return m_data[index];
	}
	
	Vector3& Vector3::operator+=(const Vector3& v)
	{
		m_data[0] += v.m_data[0];
		m_data[1] += v.m_data[1];
		m_data[2] += v.m_data[2];
		return *this;
	}
	
	Vector3& Vector3::operator-=(const Vector3& v)
	{
		m_data[0] -= v.m_data[0];
		m_data[1] -= v.m_data[1];
		m_data[2] -= v.m_data[2];
		return *this;
	}
	
	Vector3 Vector3::operator+(const Vector3& v)
	{
		return Vector3( m_data[0]+v.m_data[0],
										m_data[1]+v.m_data[1],
										m_data[2]+v.m_data[2]);
	}
	
	Vector3 Vector3::operator-(const Vector3& v)
	{
		return Vector3( m_data[0]-v.m_data[0],
										m_data[1]-v.m_data[1],
										m_data[2]-v.m_data[2]);
	}	
	
	Vector3 Vector3::operator-()
	{
		return Vector3(-m_data[0],-m_data[1],-m_data[2]);
	}
	
	Vector3 Vector3::operator/(REAL d)
	{
		return Vector3(m_data[0]/d,m_data[1]/d,m_data[2]/d);
	}		
	
	Vector3& Vector3::operator/=(REAL d)
	{
		m_data[0]/=d;
		m_data[1]/=d;
		m_data[2]/=d;
		return *this;
	}
	
	Vector3 Vector3::operator*(REAL d)
	{
		return Vector3(m_data[0]*d,m_data[1]*d,m_data[2]*d);
	}

	bool Vector3::operator==(const Vector3& v)
	{
		if (m_data[0]==v.m_data[0] && m_data[1]==v.m_data[1] && m_data[2]==v.m_data[2]) return true;
		else return false;
	}
	
	bool Vector3::operator!=(const Vector3& v)
	{
		if (m_data[0]!=v.m_data[0] || m_data[1]!=v.m_data[1] || m_data[2]!=v.m_data[2]) return true;
		else return false;
	}

	REAL Vector3::GetLength()
	{
		return sqrt(m_data[0]*m_data[0]+m_data[1]*m_data[1]+m_data[2]*m_data[2]);
	}
	
	REAL Vector3::GetSqLength()
	{
		return m_data[0]*m_data[0]+m_data[1]*m_data[1]+m_data[2]*m_data[2];
	}
	
	REAL Vector3::Dot(const Vector3& v) const
	{
		return m_data[0]*v.m_data[0] + m_data[1]*v.m_data[1] + m_data[2]*v.m_data[2];
	}
	
	Vector3 Vector3::Cross(const Vector3& v) const
	{
		return Vector3(m_data[1]*v.m_data[2]-m_data[2]*v.m_data[1], m_data[2]*v.m_data[0]-m_data[0]*v.m_data[2], m_data[0]*v.m_data[1]-m_data[1]*v.m_data[0]);
	}

	Vector3& Vector3::Normalise()
	{
		REAL m = (REAL)sqrt(m_data[0]*m_data[0] + m_data[1]*m_data[1] + m_data[2]*m_data[2]);
		m_data[0] /= m;
		m_data[1] /= m;
		m_data[2] /= m;
		return *this;
	}
	
	Vector3 Vector3::Normal() const
	{
		REAL m = (REAL)sqrt(m_data[0]*m_data[0] + m_data[1]*m_data[1] + m_data[2]*m_data[2]);
		return Vector3(m_data[0]/m,m_data[1]/m,m_data[2]/m);
	}

	Vector3& Vector3::Reverse()
	{
		m_data[0] = -1*m_data[0];
		m_data[1] = -1*m_data[1];
		m_data[2] = -1*m_data[2];
		return *this;
	}
	
	std::ostream& operator<<(std::ostream& out, Vector3& v)
	{
		return out << "[" << v[0] << "," << v[1] << "," << v[2] << "]";
	}
}

